@@ -1,8 +1,8 @@
-#!/usr/bin/php -q
+#!/usr/bin/env php
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2019 The Cacti Group                                 |
+ | Copyright (C) 2004-2022 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -35,6 +35,11 @@ require_once($config['base_path'] . '/lib/snmp.php');
 require_once($config['base_path'] . '/lib/template.php');
 require_once($config['base_path'] . '/lib/utility.php');
 
+/* switch to main database for cli's */
+if ($config['poller_id'] > 1) {
+	db_switch_remote_to_main();
+}
+
 /* process calling arguments */
 $parms = $_SERVER['argv'];
 array_shift($parms);
@@ -70,6 +75,7 @@ if (cacti_sizeof($parms)) {
 	$ping_timeout   = read_config_option('ping_timeout');
 	$ping_retries   = read_config_option('ping_retries');
 	$max_oids       = read_config_option('max_get_size');
+	$bulk_walk_size = -1;
 	$proxy          = false;
 	$device_threads = read_config_option('device_threads');;
 
@@ -267,6 +273,15 @@ if (cacti_sizeof($parms)) {
 			}
 
 			break;
+		case '--bulk_walk':
+			if (is_numeric($value) && $value >= -1 && $value != 0) {
+				$bulk_walk_size = $value;
+			} else {
+				print "ERROR: Invalid Bulk Walk Size: ($value)\n\n";
+				display_help();
+				exit(1);
+			}
+
 		case '--version':
 		case '-V':
 		case '-v':
@@ -319,7 +334,7 @@ if (cacti_sizeof($parms)) {
 
 	/* process host description */
 	if (isset($hosts[$description])) {
-		db_execute("UPDATE host SET hostname='$ip' WHERE id=" . $hosts[$description]);
+		db_execute("UPDATE host SET hostname='$ip' WHERE deleted = '' AND id=" . $hosts[$description]);
 		print "This host already exists in the database ($description) device-id: (" . $hosts[$description] . ")\n";
 		exit(1);
 	}
@@ -383,7 +398,7 @@ if (cacti_sizeof($parms)) {
 		}
 
 		if ($fail) {
-			db_execute("UPDATE host SET description = '$description' WHERE id = " . $addresses[$ip]);
+			db_execute("UPDATE host SET description = '$description' WHERE deleted = '' AND id = " . $addresses[$ip]);
 			print "ERROR: This IP already exists in the database ($ip) device-id: (" . $addresses[$ip] . ")\n";
 			exit(1);
 		}
@@ -445,7 +460,7 @@ if (cacti_sizeof($parms)) {
 		$ping_port, $ping_timeout, $ping_retries, $notes,
 		$snmp_auth_protocol, $snmp_priv_passphrase,
 		$snmp_priv_protocol, $snmp_context, $snmp_engine_id, $max_oids, $device_threads,
-		$poller_id, $site_id, $external_id, $location);
+		$poller_id, $site_id, $external_id, $location, $bulk_walk_size);
 
 	if (is_error_message()) {
 		print "ERROR: Failed to add this device\n";
@@ -504,6 +519,7 @@ function display_help() {
 	print "    --context      '', snmp context for snmpv3\n";
 	print "    --engineid     '', snmp engineid for snmpv3\n";
 	print "    --max_oids     10, 1-60, the number of OIDs that can be obtained in a single SNMP Get request\n\n";
+	print "    --bulk_walk    -1, 1-60, the bulk walk chunk size that will be used for bulk walks.  Use -1 for auto-tune.\n\n";
 	print "List Options:\n";
 	print "    --list-host-templates\n";
 	print "    --list-communities\n";
