@@ -1,8 +1,8 @@
-#!/usr/bin/php -q
+#!/usr/bin/env php
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2019 The Cacti Group                                 |
+ | Copyright (C) 2004-2022 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -32,6 +32,7 @@ array_shift($parms);
 global $debug;
 
 $debug = false;
+$local = false;
 $form  = '';
 $start = time();
 
@@ -49,6 +50,9 @@ if (cacti_sizeof($parms)) {
 			case '--debug':
 				$debug = true;
 				break;
+			case '--local':
+				$local = true;
+				break;
 			case '--version':
 			case '-V':
 			case '-v':
@@ -67,14 +71,28 @@ if (cacti_sizeof($parms)) {
 	}
 }
 
-print "Analyzing All Cacti Database Tables\n";
+print "NOTE: Analyzing All Cacti Database Tables\n";
+
+if (!$local && $config['poller_id'] > 1) {
+	db_switch_remote_to_main();
+
+	print "NOTE: Repairing Tables for Main Database" . PHP_EOL;
+} else {
+	print "NOTE: Repairing Tables for Local Database" . PHP_EOL;
+}
 
 $tables = db_fetch_assoc('SHOW TABLES FROM `' . $database_default . '`');
 
 if (cacti_sizeof($tables)) {
 	foreach($tables AS $table) {
-		print "Analyzing Table -> '" . $table['Tables_in_' . $database_default] . "'";
-		$status = db_execute('ANALYZE TABLE ' . $table['Tables_in_' . $database_default] . $form);
+		if (db_binlog_enabled()) {
+			print "NOTE: Analyzing Table -> '" . $table['Tables_in_' . $database_default] . "' without writing to the binlog";
+			$status = db_execute('ANALYZE TABLE NO_WRITE_TO_BINLOG ' . $table['Tables_in_' . $database_default] . $form);
+		} else {
+			print "NOTE: Analyzing Table -> '" . $table['Tables_in_' . $database_default] . "'";
+			$status = db_execute('ANALYZE TABLE ' . $table['Tables_in_' . $database_default] . $form);
+		}
+
 		print ($status == 0 ? ' Failed' : ' Successful') . "\n";
 	}
 
@@ -93,7 +111,8 @@ function display_help () {
 
 	print "\nusage: analyze_database.php [-d|--debug]\n\n";
 	print "A utility to recalculate the cardinality of indexes within the Cacti database.\n";
-	print "It's important to periodically run this utility expecially on larger systems.\n\n";
+	print "It's important to periodically run this utility especially  on larger systems.\n\n";
 	print "Optional:\n";
-	print "-d | --debug - Display verbose output during execution\n\n";
+	print "     --local   - Perform the action on the Remote Data Collector if run from there\n";
+	print "-d | --debug   - Display verbose output during execution\n\n";
 }
