@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2019 The Cacti Group                                 |
+ | Copyright (C) 2004-2022 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -64,17 +64,17 @@ if ($debug == false) {
 	set_request_var('rra_id', 0);
 }
 
-session_write_close();
+cacti_session_close();
 
 $graph_data_array = array();
 
 /* override: graph start time (unix time) */
-if (!isempty_request_var('graph_start') && get_request_var('graph_start') < 1600000000) {
+if (!isempty_request_var('graph_start') && get_request_var('graph_start') < FILTER_VALIDATE_MAX_DATE_AS_INT) {
 	$graph_data_array['graph_start'] = get_request_var('graph_start');
 }
 
 /* override: graph end time (unix time) */
-if (!isempty_request_var('graph_end') && get_request_var('graph_end') < 1600000000) {
+if (!isempty_request_var('graph_end') && get_request_var('graph_end') < FILTER_VALIDATE_MAX_DATE_AS_INT) {
 	$graph_data_array['graph_end'] = get_request_var('graph_end');
 }
 
@@ -263,6 +263,7 @@ if ($output !== false && $output != '' && strpos($output, 'image = ') !== false)
 }
 
 header('Content-Type: application/json');
+header('Cache-Control: max-age=15');
 $json = json_encode($oarray);
 header('Content-Length: ' . strlen($json));
 print $json;
