@@ -46,9 +46,9 @@ switch (get_request_var('action')) {
 			INNER JOIN data_template AS dt
 			ON dtr.data_template_id = dt.id
 			WHERE dtr.local_data_id = 0
-			AND dtr.data_template_id = ?
+			AND (dtr.data_template_id = ? OR dtr.id = ?)
 			ORDER BY dt.name, dtr.data_source_name",
-			array($data_template_id));
+			array($data_template_id, $task_item_id));
 
 		$output = '';
 
@@ -459,7 +459,7 @@ function item_edit() {
 		),
 	);
 
-	validate_store_request_vars($filters, 'sess_gti');
+	validate_store_request_vars($filters, 'sess_gti_' . get_filter_request_var('graph_template_id'));
 	/* ================= input validation ================= */
 
 	if (get_request_var('graph_template_id') > 0 || isset_request_var('id')) {
@@ -494,9 +494,6 @@ function item_edit() {
 					set_request_var('data_template_id', $dt);
 					break;
 				}
-				cacti_log('Not Already Set');
-			} else {
-				cacti_log('Already Set');
 			}
 		}
 	}
@@ -536,6 +533,18 @@ function item_edit() {
 		$sql_where = '';
 	}
 
+	$data_template_helper = array(
+		'data_template_id' => array(
+			'friendly_name' => __('Data Template Filter'),
+			'method' => 'drop_sql',
+			'sql' => 'SELECT id, name FROM data_template ORDER BY name',
+			'default' => '0',
+			'value' => (isset_request_var('data_template_id') ? get_filter_request_var('data_template_id'):'0'),
+			'none_value' => __('Any'),
+			'description' => __('This filter will limit the Data Sources visible in the Data Source dropdown.')
+		)
+	);
+
 	/* modifications to the default graph items array */
 	$struct_graph_item['task_item_id']['sql'] = "SELECT dtr.id,
 		CONCAT_WS('', dt.name,' - ',' (', dtr.data_source_name,')') AS name
@@ -546,10 +555,12 @@ function item_edit() {
 		$sql_where
 		ORDER BY dt.name, dtr.data_source_name";
 
+	$mystruct_graph_item = array_merge($data_template_helper, $struct_graph_item);
+
 	$form_array = array();
 
-	foreach ($struct_graph_item as $field_name => $field_array) {
-		$form_array += array($field_name => $struct_graph_item[$field_name]);
+	foreach ($mystruct_graph_item as $field_name => $field_array) {
+		$form_array += array($field_name => $mystruct_graph_item[$field_name]);
 
 		if ($field_name != 'data_template_id') {
 			$form_array[$field_name]['value']   = (isset($template_item) ? $template_item[$field_name] : '');
