@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2021 The Cacti Group                                 |
+ | Copyright (C) 2004-2022 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -116,8 +116,13 @@ function store_get_selected_dq_index($snmp_query_id) {
 
 function form_save() {
 	if (isset_request_var('save_component_graph')) {
+		$form_data = array();
+
 		/* summarize the 'create graph from host template/snmp index' stuff into an array */
 		foreach ($_POST as $var => $val) {
+			/* save form data */
+			$form_data[$var] = $val;
+
 			if (preg_match('/^cg_(\d+)$/', $var, $matches)) {
 				$selected_graphs['cg'][$matches[1]][$matches[1]] = true;
 			} elseif (preg_match('/^cg_g$/', $var)) {
@@ -134,6 +139,12 @@ function form_save() {
 			}
 		}
 
+		/* save the json_encoded form data in case of an error */
+		$form_data['header'] = false;
+
+		$_SESSION['sess_graphs_new_form'] = json_encode($form_data);
+		$_SESSION['sess_grn_returnto']    = 'graphs_new.php';
+
 		if (!isset_request_var('host_id')) {
 			$host_id = 0;
 		} else {
@@ -182,6 +193,7 @@ function host_new_graphs_save($host_id) {
 	$selected_graphs_array = unserialize(stripslashes(get_nfilter_request_var('selected_graphs_array')));
 
 	$values = array();
+	$form_data = array();
 
 	/* form an array that contains all of the data on the previous form */
 	foreach ($_POST as $var => $val) {
@@ -219,7 +231,21 @@ function host_new_graphs_save($host_id) {
 			} else { // this is a data query field
 				$values['sg'][$matches[1]][$matches[2]]['data_template'][$matches[3]][$matches[4]] = $val;
 			}
-		} elseif (preg_match('/^c_(\d+)_(\d+)_(\d+)_(\d+)/', $var, $matches)) { // 1: snmp_query_id, 2: graph_template_id, 3: data_template_id, 4:data_input_field_id
+		} elseif (preg_match('/^c_(\d+)_(\d+)_(\d+)_(\d+)/', $var, $matches)) {
+			/**
+			 * Custom Data.  Need to validate these against the input regular expressions
+			 *
+			 * Index offsets
+			 * 1: snmp_query_id, 2: graph_template_id, 3: data_template_id, 4:data_input_field_id
+			 */
+			$input_field_id = $matches[4];
+			$idata = db_fetch_row_prepared('SELECT *
+				FROM data_input_fields
+				WHERE id = ?',
+				array($input_field_id));
+
+			$val = form_input_validate($val, $var, $idata['regexp_match'], $idata['allow_nulls'], 3);
+
 			if (empty($matches[1])) { // this is a new graph from template field
 				$values['cg'][$matches[2]]['custom_data'][$matches[3]][$matches[4]] = $val;
 			} else { // this is a data query field
@@ -234,15 +260,32 @@ function host_new_graphs_save($host_id) {
 		}
 	}
 
-	debug_log_clear('new_graphs');
+	if (!is_error_message()) {
+		debug_log_clear('new_graphs');
 
-	foreach ($selected_graphs_array as $form_type => $form_array) {
-		$current_form_type = $form_type;
+		foreach ($selected_graphs_array as $form_type => $form_array) {
+			$current_form_type = $form_type;
 
-		foreach ($form_array as $form_id1 => $form_array2) {
-			/* enumerate information from the arrays stored in post variables */
-			create_save_graph($host_id, $form_type, $form_id1, $form_array2, $values);
+			foreach ($form_array as $form_id1 => $form_array2) {
+				/* enumerate information from the arrays stored in post variables */
+				create_save_graph($host_id, $form_type, $form_id1, $form_array2, $values);
+			}
 		}
+	} else {
+		$form_data = $_SESSION['sess_graphs_new_form'];
+		kill_session_var('sess_graphs_new_form');
+
+		?>
+		<script type='text/javascript'>
+		var formData=<?php print $form_data;?>;
+
+		$(function() {
+			loadPageUsingPost('graphs_new.php', formData);
+		});
+		</script>
+		<?php
+
+		exit;
 	}
 }
 
@@ -617,24 +660,28 @@ function graphs() {
 
 				$xml_array = get_data_query_array($snmp_query['id']);
 
-				$num_input_fields = 0;
+				$num_input_fields   = 0;
 				$num_visible_fields = 0;
 
 				if (cacti_sizeof($xml_array)) {
 					/* loop through once so we can find out how many input fields there are */
-					foreach ($xml_array['fields'] as $field_name => $field_array) {
-						if ($field_array['direction'] == 'input' || $field_array['direction'] == 'input-output') {
-							$num_input_fields++;
+					if (isset($xml_array['fields'])) {
+						foreach ($xml_array['fields'] as $field_name => $field_array) {
+							if ($field_array['direction'] == 'input' || $field_array['direction'] == 'input-output') {
+								$num_input_fields++;
 
-							if (!isset($total_rows)) {
-								$total_rows = db_fetch_cell_prepared('SELECT count(*)
-									FROM host_snmp_cache
-									WHERE host_id = ?
-									AND snmp_query_id = ?
-									AND field_name = ?',
-									array($host['id'], $snmp_query['id'], $field_name));
+								if (!isset($total_rows)) {
+									$total_rows = db_fetch_cell_prepared('SELECT COUNT(*)
+										FROM host_snmp_cache
+										WHERE host_id = ?
+										AND snmp_query_id = ?
+										AND field_name = ?',
+										array($host['id'], $snmp_query['id'], $field_name));
+								}
 							}
 						}
+					} else {
+						raise_message('xmlerror', __('Error Parsing Data Query Resource XML file for Data Query \'%s\' with id \'%s\'', $snmp_query['name'], $snmp_query['id']), MESSAGE_LEVEL_ERROR);
 					}
 				}
 
@@ -672,7 +719,7 @@ function graphs() {
 
 				print "<div class='cactiTable'><div><div class='cactiTableTitle'><span>" . __esc('Data Query [%s]', $snmp_query['name']) . "</span></div><div class='cactiTableButton'><span class='reloadquery fa fa-sync' id='reload" . $snmp_query['id'] . "' data-id='" . $snmp_query['id'] . "'></span></div></div></div>";
 
-				if ($xml_array != false) {
+				if (cacti_sizeof($xml_array)) {
 					$html_dq_header = '';
 
 					/* if there is a where clause, get the matching snmp_indexes */
@@ -795,21 +842,38 @@ function graphs() {
 								</tr>";
 						}
 
+						/* disable graph creation if there are no associated Graph Templates */
+						$enabled = db_fetch_cell_prepared('SELECT COUNT(*)
+							FROM snmp_query_graph
+							WHERE snmp_query_id = ?',
+							array($snmp_query['id']));
+
+						if (!$enabled) {
+							$disabled_text = __esc('The index is disabled due to the Data Query having no associated Graph Templates.');
+						} else {
+							$disabled_text = '';
+						}
+
 						$row_counter    = 0;
 						$column_counter = 0;
 						$fields         = array_rekey($field_names, 'field_name', 'field_name');
+
 						if (cacti_sizeof($snmp_query_indexes)) {
 							foreach($snmp_query_indexes as $row) {
 								$query_row = $snmp_query['id'] . '_' . encode_data_query_index($row['snmp_index']);
 
-								print "<tr id='dqline$query_row' class='selectable " . (($row_counter % 2 == 0) ? 'odd' : 'even') . "'>"; $i++;
+								if ($enabled) {
+									print "<tr id='dqline$query_row' class='selectable " . (($row_counter % 2 == 0) ? 'odd' : 'even') . "'>"; $i++;
+								} else {
+									print "<tr id='nodqline$query_row' title='$disabled_text' class='selectable notemplate " . (($row_counter % 2 == 0) ? 'odd' : 'even') . "'>"; $i++;
+								}
 
 								$column_counter = 0;
 								foreach ($xml_array['fields'] as $field_name => $field_array) {
 									if ($field_array['direction'] == 'input' || $field_array['direction'] == 'input-output') {
 										if (in_array($field_name, $fields)) {
 											if (isset($row[$field_name])) {
-												print "<td title='" . html_escape($row[$field_name]) . "'><span class='textOverflow' id='text$query_row" . '_' . $column_counter . "'>" . filter_value($row[$field_name], get_request_var('filter')) . '</span></td>';
+												print "<td><span class='textOverflow' id='text$query_row" . '_' . $column_counter . "'>" . filter_value($row[$field_name], get_request_var('filter')) . '</span></td>';
 											} else {
 												print "<td><span class='textOverflow' id='text$query_row" . '_' . $column_counter . "'></span></td>";
 											}
@@ -820,7 +884,12 @@ function graphs() {
 								}
 
 								print "<td style='width:1%;' class='checkbox'>";
-								print "<input class='checkbox' type='checkbox' name='sg_$query_row' id='sg_$query_row'><label class='formCheckboxLabel' for='sg_$query_row'></label>";
+
+								if ($enabled) {
+									print "<input class='checkbox' type='checkbox' name='sg_$query_row' id='sg_$query_row'><label class='formCheckboxLabel' for='sg_$query_row'></label>";
+								} else {
+									print "<input class='checkbox' type='checkbox' disabled name='sg_$query_row' id='sg_$query_row'><label class='formCheckboxLabel' for='sg_$query_row'></label>";
+								}
 								print '</td>';
 								print '</tr>';
 
@@ -835,7 +904,7 @@ function graphs() {
 				} else {
 					html_start_box('', '100%', '', '3', 'center', '');
 
-					print "<tr class='odd'><td class='textError'>" . __('Error in data query.') . '</td></tr>';
+					print "<tr class='odd'><td class='textError'>" . __('Error in Data Query.  This could be due to the following: File Permissions, or a missing or improperly formatted Data Query XML file.') . '</td></tr>';
 				}
 
 				html_end_box(false);
@@ -882,7 +951,7 @@ function graphs() {
 	}
 
 	if ($script != '') {
-		$script .= "$('.default').click(function() { $.get('graphs_new.php?action=ajax_save&query=" . (isset($snmp_query['id']) ? $snmp_query['id']:'') . "'+'&item='+$(\".dqselect\").val()).fail(function(data) { getPresentHTTPError(data); });});</script>";
+		$script .= "$('.default').click(function() { $.get('graphs_new.php?action=ajax_save&query=" . (isset($snmp_query['id']) ? $snmp_query['id']:'') . "'+'&item='+$(\".dqselect\").val()).fail(function(data) { getPresentHTTPError(data); });}); $('tr.notemplate').tooltip();</script>";
 		print $script;
 	}
 
@@ -893,12 +962,13 @@ function graphs() {
 		form_hidden_box('host_template_id', $host['host_template_id'], '0');
 	}
 
-	if (isset($_SERVER['HTTP_REFERER']) && !substr_count($_SERVER['HTTP_REFERER'], 'graphs_new')) {
+	if (isset($_SERVER['HTTP_REFERER']) && strpos($_SERVER['HTTP_REFERER'], 'graphs_new') === false) {
 		set_request_var('returnto', basename($_SERVER['HTTP_REFERER']));
 	}
 
 	load_current_session_value('returnto', 'sess_grn_returnto', '');
-	if (substr_count(get_nfilter_request_var('returnto'), 'host.php') == 0) {
+
+	if (strpos(get_nfilter_request_var('returnto'), 'host.php') === false) {
 		set_request_var('returnto', '');
 	}
 
