@@ -1,8 +1,13 @@
+// Host Autocomplete Magic
+var pageName = basename($(location).attr('pathname'));
+
 function themeReady() {
 	height = get_height();
 	$('#navigation, .cactiConsoleNavigationArea').css('height', height);
 	$('#navigation, #navigation_right').show();
 
+	keepWindowSize();
+
 	$(window).unbind().resize(function(event) {
 		if (pageName == 'graph_view.php') {
 			treeWidth    = $('#navigation').width();
