@@ -22,18 +22,26 @@
  +-------------------------------------------------------------------------+
 */
 
-/* api_device_crc_update - update hash stored in settings table to inform
-   remote pollers to update their caches
-   @arg $poller_id - the id of the poller impacted by hash update
-   @arg $variable  - the hash variable prefix for the replication setting. */
+/**
+ * api_device_crc_update - update hash stored in settings table to inform
+ * remote pollers to update their caches
+ *
+ * @param  (int)    The id of the poller impacted by hash update
+ * @param  (string) The hash variable prefix for the replication setting.
+ *
+ * @return (void)
+ */
 function api_device_cache_crc_update($poller_id, $variable = 'poller_replicate_device_cache_crc') {
 	$hash = hash('ripemd160', date('Y-m-d H:i:s') . rand() . $poller_id);
 
 	db_execute_prepared("REPLACE INTO settings SET value = ?, name='$variable" . "_" . "$poller_id'", array($hash));
 }
 
-/* api_device_remove - removes a device
-   @arg $device_id - the id of the device to remove */
+/**
+ * api_device_remove - removes a device
+ *
+ * @param  $device_id - the id of the device to remove
+ */
 function api_device_remove($device_id) {
 	global $config;
 
@@ -77,9 +85,12 @@ function api_device_remove($device_id) {
 	api_device_cache_crc_update($poller_id);
 }
 
-/* api_device_purge_from_remote - removes a device from a remote data collectors
-   @arg $device_ids - device id or an array of device_ids of a host or hosts
-   @arg $poller_id  - the previous poller if it changed */
+/**
+ * api_device_purge_from_remote - removes a device from a remote data collectors
+ *
+ * @param  $device_ids - device id or an array of device_ids of a host or hosts
+ * @param  $poller_id  - the previous poller if it changed
+ */
 function api_device_purge_from_remote($device_ids, $poller_id = 0) {
 	if (!is_array($device_ids)) {
 		$device_ids = array($device_ids);
@@ -119,6 +130,12 @@ function api_device_purge_from_remote($device_ids, $poller_id = 0) {
 	}
 }
 
+/**
+ * api_device_purge_deleted_devices - Remove any devices from the database that are
+ *   marked for deletion.
+ *
+ * @return (void)
+ */
 function api_device_purge_deleted_devices() {
 	$devices = db_fetch_assoc_prepared('SELECT id, poller_id
 		FROM host
@@ -144,9 +161,14 @@ function api_device_purge_deleted_devices() {
 	}
 }
 
-/* api_device_remove_multi - removes multiple devices in one call
-   @arg $device_ids - an array of device id's to remove
-   @arg $delete_type - boolean to keep data source and graphs or remove */
+/**
+ * api_device_remove_multi - removes multiple devices in one call
+ *
+ * @param  (array) An array of device id's to remove
+ * @param  (int)   Boolean to keep data source and graphs or remove
+ *
+ * @return (void)
+ */
 function api_device_remove_multi($device_ids, $delete_type = 2) {
 	global $config;
 
@@ -224,6 +246,13 @@ function api_device_remove_multi($device_ids, $delete_type = 2) {
 	}
 }
 
+/**
+ * api_device_disable_devices - Disable an array of device ids
+ *
+ * @param  (array) An array of device ids
+ *
+ * @return (void)
+ */
 function api_device_disable_devices($device_ids) {
 	global $config;
 
@@ -258,6 +287,13 @@ function api_device_disable_devices($device_ids) {
 	}
 }
 
+/**
+ * api_device_enable_devices - Enable an array of device ids
+ *
+ * @param  (array) An array of device ids
+ *
+ * @return (void)
+ */
 function api_device_enable_devices($device_ids) {
 	global $config;
 
@@ -306,9 +342,10 @@ function api_device_enable_devices($device_ids) {
 				array($device_id));
 		}
 
-		// Only reprime poller cache if empty
-		// this allows support for pre 1.2.16
-		// enable behavior.
+		/**
+		 * Only reprime poller cache if empty this allows support for pre 1.2.16
+		 * enable behavior.
+		 */
 		if (!cacti_sizeof($poller_cache)) {
 			/* update poller cache */
 			$data_sources = db_fetch_assoc_prepared('SELECT id
@@ -346,6 +383,15 @@ function api_device_enable_devices($device_ids) {
 	}
 }
 
+/**
+ * api_device_change_options - Given an array of devive ids and the
+ *   post variable, update a series of Device settings.
+ *
+ * @param  (array) An array of device ids
+ * @param  (array) An array representing the $_POST variable
+ *
+ * @return (void)
+ */
 function api_device_change_options($device_ids, $post) {
 	global $config, $fields_host_edit;
 
@@ -414,6 +460,14 @@ function api_device_change_options($device_ids, $post) {
 	}
 }
 
+/**
+ * api_device_clear_statistics - Clear all device level statistics and reset as if the
+ *   device was new in Cacti
+ *
+ * @param  (array) An array of device ids
+ *
+ * @return (void)
+ */
 function api_device_clear_statistics($device_ids) {
 	global $config;
 
@@ -450,6 +504,14 @@ function api_device_clear_statistics($device_ids) {
 	}
 }
 
+/**
+ * api_device_sync_device_templates - Sync an array of device ids with their
+ *   parent Device Template
+ *
+ * @param (array) An array of device ids
+ *
+ * @return (void)
+ */
 function api_device_sync_device_templates($device_ids) {
 	global $config;
 
@@ -465,10 +527,14 @@ function api_device_sync_device_templates($device_ids) {
 	}
 }
 
-/* api_device_dq_add - adds a device->data query mapping
-   @arg $device_id - the id of the device which contains the mapping
-   @arg $data_query_id - the id of the data query to remove the mapping for
-   @arg $reindex_method - the reindex method to user when adding the data query */
+/**
+ * api_device_dq_add - adds a device->data query mapping
+ * @param  (int)  The id of the device which contains the mapping
+ * @param  (int)  The id of the data query to remove the mapping for
+ * @param  (int)  The reindex method to user when adding the data query
+ *
+ * @return (void)
+ */
 function api_device_dq_add($device_id, $data_query_id, $reindex_method) {
 	global $config;
 
@@ -498,9 +564,14 @@ function api_device_dq_add($device_id, $data_query_id, $reindex_method) {
 	run_data_query($device_id, $data_query_id);
 }
 
-/* api_device_dq_remove - removes a device->data query mapping
-   @arg $device_id - the id of the device which contains the mapping
-   @arg $data_query_id - the id of the data query to remove the mapping for */
+/**
+ * api_device_dq_remove - removes a device->data query mapping
+ *
+ * @param  (int) The id of the device which contains the mapping
+ * @param  (int) The id of the data query to remove the mapping for
+ *
+ * @return (void)
+ */
 function api_device_dq_remove($device_id, $data_query_id) {
 	global $config;
 
@@ -547,10 +618,15 @@ function api_device_dq_remove($device_id, $data_query_id) {
 	}
 }
 
-/* api_device_dq_change - changes a device->data query mapping
-   @arg $device_id - the id of the device which contains the mapping
-   @arg $data_query_id - the id of the data query to remove the mapping for
-   @arg $reindex_method - the reindex method to use when changing the data query */
+/**
+ * api_device_dq_change - changes a device->data query mapping
+ *
+ * @param  (int) The id of the device which contains the mapping
+ * @param  (int) The id of the data query to remove the mapping for
+ * @param  (int) The reindex method to use when changing the data query
+ *
+ * @return (void)
+ */
 function api_device_dq_change($device_id, $data_query_id, $reindex_method) {
 	global $config;
 
@@ -590,9 +666,14 @@ function api_device_dq_change($device_id, $data_query_id, $reindex_method) {
 	run_data_query($device_id, $data_query_id);
 }
 
-/* api_device_gt_remove - removes a device->graph template mapping
-   @arg $device_id - the id of the device which contains the mapping
-   @arg $graph_template_id - the id of the graph template to remove the mapping for */
+/**
+ * api_device_gt_remove - removes a device->graph template mapping
+ *
+ * @param  (int) The id of the device which contains the mapping
+ * @param  (int) The id of the graph template to remove the mapping for
+ *
+ * @return (void)
+ */
 function api_device_gt_remove($device_id, $graph_template_id) {
 	global $config;
 
@@ -619,6 +700,14 @@ function api_device_gt_remove($device_id, $graph_template_id) {
 	}
 }
 
+/**
+ * api_device_replicate_out - Replace device settings to the remote data collectors
+ *
+ * @param  (int) The id of the device
+ * @param  (int) The poller id of the device.  If null, we determine it
+ *
+ * @return (void)
+ */
 function api_device_replicate_out($device_id, $poller_id = 1) {
 	global $config;
 
@@ -794,7 +883,46 @@ function api_device_replicate_out($device_id, $poller_id = 1) {
 	return true;
 }
 
-function api_device_save($id, $host_template_id, $description, $hostname, $snmp_community, $snmp_version,
+/**
+ * api_device_save - Save a device and update the poller cache for the device is required.
+ *   The function will determine if the poller cache needs updating by reviewing the changed
+ *   settings.  If no settings changed that require an update of the poller cache, the
+ *   device level settings will simply be updated, otherwise the poller cache will be refreshed
+ *   for the device.
+ *
+ * @param  (int)    The id of the device
+ * @param  (int)    The device template for the device
+ * @param  (string) A device description
+ * @param  (string) The devices hostname
+ * @param  (string) The devices snmp community in the case of v1/v2c
+ * @param  (int)    The devices snmp_version 1|2|3
+ * @param  (string) The devices snmp username in the case of v3
+ * @param  (string) The devices snmp auth password in the case of v3
+ * @param  (int)    The devices snmp port if in use.  Default to 161
+ * @param  (int)    The devices snmp timeout in milliseconds
+ * @param  (bool)   True of 'on' if the device is disabled
+ * @param  (int)    The devices availability/reachability type
+ * @param  (int)    The devices availability/reachability test ping method
+ * @param  (int)    The devices ping port to be used in the case of TCP or UDP
+ * @param  (int)    The ping timeout in milliseconds
+ * @param  (int)    The number of times to retry the ping of the device
+ * @param  (strong) Operator notes for the device.  Can be used by plugins
+ * @param  (int)    The snmp authentication protocol
+ * @param  (string) The snmp privilege protocol passphrase
+ * @param  (int)    The snmp privilege protocol to use
+ * @param  (string) The snmp context to use to reach the device
+ * @param  (string) The snmp engine id if required to reach the devices
+ * @param  (int)    The maximum number of OID's to gather in a single snmpget request
+ * @param  (int)    When using spine, the number of threads to use to collect data source information
+ * @param  (int)    The id of the data collector.  The default is 1
+ * @param  (int)    The id of the site that the device belongs to
+ * @param  (string) External ID's to be used by plugins and other cmdb like functions
+ * @param  (string) A location attribute such as rack and enclosure, closet location within a site.
+ * @param  (int)    A variable that tells cacti to find detect the optimal bulk walk size for the device
+ *
+ * @return (int)    The id of the device
+ */
+function api_device_save($id, $device_template_id, $description, $hostname, $snmp_community, $snmp_version,
 	$snmp_username, $snmp_password, $snmp_port, $snmp_timeout, $disabled,
 	$availability_method, $ping_method, $ping_port, $ping_timeout, $ping_retries,
 	$notes, $snmp_auth_protocol, $snmp_priv_passphrase, $snmp_priv_protocol, $snmp_context, $snmp_engine_id,
@@ -827,7 +955,7 @@ function api_device_save($id, $host_template_id, $description, $hostname, $snmp_
 	$raised = false;
 
 	$save['id']                   = form_input_validate($id, 'id', '^[0-9]+$', false, 3);
-	$save['host_template_id']     = form_input_validate($host_template_id, 'host_template_id', '^[0-9]+$', true, 3);
+	$save['host_template_id']     = form_input_validate($device_template_id, 'host_template_id', '^[0-9]+$', true, 3);
 
 	$save['poller_id']            = form_input_validate($poller_id, 'poller_id', '^[0-9]+$', true, 3);
 	$save['site_id']              = form_input_validate($site_id, 'site_id', '^[0-9]+$', true, 3);
@@ -888,17 +1016,17 @@ function api_device_save($id, $host_template_id, $description, $hostname, $snmp_
 	$save['bulk_walk_size']       = form_input_validate($bulk_walk_size, 'bulk_walk_size', '^[-0-9]+$', false, 3);
 	$save['device_threads']       = form_input_validate($device_threads, 'device_threads', '^[0-9]+$', true, 3);
 
-	$host_id = 0;
+	$device_id = 0;
 
 	if (!is_error_message()) {
 		$save = api_plugin_hook_function('api_device_save', $save);
 
-		$host_id = sql_save($save, 'host');
+		$device_id = sql_save($save, 'host');
 
-		if ($host_id) {
-			if ($poller_id != $previous_poller) {
+		if ($device_id) {
+			if ($previous_poller > 1 && $poller_id != $previous_poller) {
 				if (remote_poller_up($previous_poller)) {
-					api_device_purge_from_remote($host_id, $previous_poller);
+					api_device_purge_from_remote($device_id, $previous_poller);
 				} else {
 					raise_message('poller_down_' . $save['id'], __('Remote Poller %s is Down, you will need to perform a FullSync once it is up again', $previous_poller), MESSAGE_LEVEL_WARN);
 					$raised = true;
@@ -909,8 +1037,8 @@ function api_device_save($id, $host_template_id, $description, $hostname, $snmp_
 
 			if ($poller_id > 1) {
 				if (remote_poller_up($poller_id)) {
-					if (($rcnn_id = poller_push_to_remote_db_connect($host_id)) !== false) {
-						$save['id'] = $host_id;
+					if (($rcnn_id = poller_push_to_remote_db_connect($device_id)) !== false) {
+						$save['id'] = $device_id;
 						sql_save($save, 'host', 'id', true, $rcnn_id);
 					} elseif (!$raised) {
 						raise_message('poller_down_' . $save['id'], __('Remote Poller %s is Down, you will need to perform a FullSync once it is up again', $poller_id), MESSAGE_LEVEL_WARN);
@@ -923,8 +1051,8 @@ function api_device_save($id, $host_template_id, $description, $hostname, $snmp_
 			}
 
 			/* clear the host session data for poller cache repopulation */
-			if (isset($_SESSION['sess_host_cache_array'][$host_id])) {
-				unset($_SESSION['sess_host_cache_array'][$host_id]);
+			if (isset($_SESSION['sess_host_cache_array'][$device_id])) {
+				unset($_SESSION['sess_host_cache_array'][$device_id]);
 			}
 
 			/* change reindex method for 'None' for non-snmp devices */
@@ -932,23 +1060,23 @@ function api_device_save($id, $host_template_id, $description, $hostname, $snmp_
 				db_execute_prepared('UPDATE host_snmp_query
 					SET reindex_method = 0
 					WHERE host_id = ?',
-					array($host_id));
+					array($device_id));
 
 				db_execute_prepared('DELETE FROM poller_reindex
 					WHERE host_id = ?',
-					array($host_id));
+					array($device_id));
 
 				if ($poller_id > 1) {
 					if (remote_poller_up($poller_id)) {
-						if (($rcnn_id = poller_push_to_remote_db_connect($host_id)) !== false) {
+						if (($rcnn_id = poller_push_to_remote_db_connect($device_id)) !== false) {
 							db_execute_prepared('UPDATE host_snmp_query
 								SET reindex_method = 0
 								WHERE host_id = ?',
-								array($host_id), true, $rcnn_id);
+								array($device_id), true, $rcnn_id);
 
 							db_execute_prepared('DELETE FROM poller_reindex
 								WHERE host_id = ?',
-								array($host_id), true, $rcnn_id);
+								array($device_id), true, $rcnn_id);
 						} elseif (!$raised) {
 							raise_message('poller_down_' . $save['id'], __('Remote Poller %s is Down, you will need to perform a FullSync once it is up again', $poller_id), MESSAGE_LEVEL_WARN);
 						}
@@ -962,28 +1090,28 @@ function api_device_save($id, $host_template_id, $description, $hostname, $snmp_
 
 			/* push out relavant fields to data sources using this host */
 			if (!$quick_save) {
-				push_out_host($host_id, 0);
+				push_out_host($device_id, 0);
 			}
 
 			/* the host substitution cache is now stale; purge it */
 			kill_session_var('sess_host_cache_array');
 
 			/* update title cache for graph and data source */
-			update_data_source_title_cache_from_host($host_id);
-			update_graph_title_cache_from_host($host_id);
+			update_data_source_title_cache_from_host($device_id);
+			update_graph_title_cache_from_host($device_id);
 		} else {
 			raise_message(2);
 		}
 
 		/* if the user changes the host template, add each snmp query associated with it */
-		if ($host_template_id > 0 && $host_template_id != $_host_template_id) {
-			api_device_update_host_template($host_id, $host_template_id);
+		if ($device_template_id > 0 && $device_template_id != $_host_template_id) {
+			api_device_update_host_template($device_id, $device_template_id);
 		}
 	}
 
-	if ($host_id > 0) {
+	if ($device_id > 0) {
 		if (read_config_option('extended_paths') == 'on'){
-			$host_dir = $config['rra_path'] . '/' . $host_id;
+			$host_dir = $config['rra_path'] . '/' . $device_id;
 			if (!is_dir($host_dir)){
 				if (is_writable($config['rra_path'])) {
 					if (mkdir($host_dir, 0775)) {
@@ -1008,22 +1136,26 @@ function api_device_save($id, $host_template_id, $description, $hostname, $snmp_
 		}
 
 		# now that we have the id of the new host, we may plugin postprocessing code
-		$save['id'] = $host_id;
+		$save['id'] = $device_id;
 
 		snmpagent_api_device_new($save);
 
-		automation_execute_device_create_tree($host_id);
+		automation_execute_device_create_tree($device_id);
 
 		api_plugin_hook_function('api_device_new', $save);
 	}
 
-	return $host_id;
+	return $device_id;
 }
 
-/* api_device_quick_save - checks if the poller cache needs to be
-   rebuilt as a part of a device save.
-   @arg $save - the save structure for the device
-   @returns boolean */
+/**
+ * api_device_quick_save - checks if the poller cache needs to be
+ *   rebuilt as a part of a device save.
+ *
+ * @param  (array) The devices "save" structure for the device
+ *
+ * @return (bool)  If the device can be quickly saved, or will the device have to be pushed out
+ */
 function api_device_quick_save(&$save) {
 	if ($save['id'] > 0) {
 		$device = db_fetch_row_prepared('SELECT *
@@ -1060,10 +1192,15 @@ function api_device_quick_save(&$save) {
 	}
 }
 
-/* api_device_update_host_template - changes the host template of a host
-   @arg $host_id - the id of the device which contains the mapping
-   @arg $host_template_id - the id of the host template alter the device to */
-function api_device_update_host_template($host_id, $host_template_id) {
+/**
+ * api_device_update_host_template - changes the host template of a host
+ *
+ * @param  (int)  The id of the device which contains the mapping
+ * @param  (int)  The id of the device template alter the device to
+ *
+ * @return (void)
+ */
+function api_device_update_host_template($device_id, $device_template_id) {
 	global $config;
 
 	static $raised = array();
@@ -1072,18 +1209,18 @@ function api_device_update_host_template($host_id, $host_template_id) {
 		SET host_template_id = ?
 		WHERE id = ?
 		AND deleted = ""',
-		array($host_template_id, $host_id));
+		array($device_template_id, $device_id));
 
 	$poller_id = db_fetch_cell_prepared('SELECT poller_id FROM host WHERE id = ?', array($device_id));
 
 	if ($poller_id > 1) {
 		if (remote_poller_up($poller_id)) {
-			if (($rcnn_id = poller_push_to_remote_db_connect($host_id)) !== false) {
+			if (($rcnn_id = poller_push_to_remote_db_connect($device_id)) !== false) {
 				db_execute_prepared('UPDATE host
 					SET host_template_id = ?
 					WHERE id = ?
 					AND deleted = ""',
-					array($host_template_id, $host_id), true, $rcnn_id);
+					array($device_template_id, $device_id), true, $rcnn_id);
 			} elseif (!isset($raised[$poller_id])) {
 				raise_message('poller_down_' . $poller_id, __('Remote Poller %s is Down, you will need to perform a FullSync once it is up again', $poller_id), MESSAGE_LEVEL_WARN);
 				$raised[$poller_id] = true;
@@ -1099,22 +1236,22 @@ function api_device_update_host_template($host_id, $host_template_id) {
 		FROM host_template_snmp_query AS htsq
 		WHERE host_template_id = ?
 		AND htsq.snmp_query_id NOT IN (SELECT snmp_query_id FROM host_snmp_cache WHERE host_id = ?)',
-		array($host_template_id, $host_id));
+		array($device_template_id, $device_id));
 
 	if (cacti_sizeof($snmp_queries)) {
 		foreach ($snmp_queries as $snmp_query) {
 			db_execute_prepared('REPLACE INTO host_snmp_query
 				(host_id, snmp_query_id, reindex_method)
 				VALUES (?, ?, ?)',
-				array($host_id, $snmp_query['snmp_query_id'], read_config_option('reindex_method')));
+				array($device_id, $snmp_query['snmp_query_id'], read_config_option('reindex_method')));
 
 			if ($poller_id > 1) {
 				if (remote_poller_up($poller_id)) {
-					if (($rcnn_id = poller_push_to_remote_db_connect($host_id)) !== false) {
+					if (($rcnn_id = poller_push_to_remote_db_connect($device_id)) !== false) {
 						db_execute_prepared('REPLACE INTO host_snmp_query
 							(host_id, snmp_query_id, reindex_method)
 							VALUES (?, ?, ?)',
-							array($host_id, $snmp_query['snmp_query_id'], read_config_option('reindex_method')), true, $rcnn_id);
+							array($device_id, $snmp_query['snmp_query_id'], read_config_option('reindex_method')), true, $rcnn_id);
 					} elseif ($raised[$poller_id]) {
 						raise_message('poller_down_' . $poller_id, __('Remote Poller %s is Down, you will need to perform a FullSync once it is up again', $poller_id), MESSAGE_LEVEL_WARN);
 						$raised[$poller_id] = true;
@@ -1126,7 +1263,7 @@ function api_device_update_host_template($host_id, $host_template_id) {
 			}
 
 			/* recache snmp data */
-			run_data_query($host_id, $snmp_query['snmp_query_id']);
+			run_data_query($device_id, $snmp_query['snmp_query_id']);
 		}
 	}
 
@@ -1135,22 +1272,22 @@ function api_device_update_host_template($host_id, $host_template_id) {
 		FROM host_template_graph AS hg
 		WHERE host_template_id = ?
 		AND hg.graph_template_id NOT IN (SELECT graph_template_id FROM host_graph WHERE host_id = ?)',
-		array($host_template_id, $host_id));
+		array($device_template_id, $device_id));
 
 	if (cacti_sizeof($graph_templates)) {
 		foreach ($graph_templates as $graph_template) {
 			db_execute_prepared('REPLACE INTO host_graph
 				(host_id, graph_template_id)
 				VALUES (?, ?)',
-				array($host_id, $graph_template['graph_template_id']));
+				array($device_id, $graph_template['graph_template_id']));
 
 			if ($poller_id > 1) {
 				if (remote_poller_up($poller_id)) {
-					if (($rcnn_id = poller_push_to_remote_db_connect($host_id)) !== false) {
+					if (($rcnn_id = poller_push_to_remote_db_connect($device_id)) !== false) {
 						db_execute_prepared('REPLACE INTO host_graph
 							(host_id, graph_template_id)
 							VALUES (?, ?)',
-							array($host_id, $graph_template['graph_template_id']), true, $rcnn_id);
+							array($device_id, $graph_template['graph_template_id']), true, $rcnn_id);
 					} elseif (!isset($raised[$poller_id])) {
 						raise_message('poller_down_' . $poller_id, __('Remote Poller %s is Down, you will need to perform a FullSync once it is up again', $poller_id), MESSAGE_LEVEL_WARN);
 						$raised[$poller_id] = true;
@@ -1161,10 +1298,10 @@ function api_device_update_host_template($host_id, $host_template_id) {
 				}
 			}
 
-			automation_hook_graph_template($host_id, $graph_template['graph_template_id']);
+			automation_hook_graph_template($device_id, $graph_template['graph_template_id']);
 
 			api_plugin_hook_function('add_graph_template_to_host',
-				array('host_id' => $host_id, 'graph_template_id' => $graph_template['graph_template_id']));
+				array('host_id' => $device_id, 'graph_template_id' => $graph_template['graph_template_id']));
 		}
 	}
 
@@ -1188,7 +1325,7 @@ function api_device_update_host_template($host_id, $host_template_id) {
 		WHERE gt.id NOT IN (SELECT graph_template_id FROM snmp_query_graph)
 	    HAVING gtid IS NULL
 	    ORDER BY gt.name',
-	    array($host_id, $host_template_id)
+	    array($device_id, $device_template_id)
 	);
 
 	if (cacti_sizeof($unused_graph_templates)) {
@@ -1197,21 +1334,21 @@ function api_device_update_host_template($host_id, $host_template_id) {
 				FROM host_graph
 				WHERE host_id = ?
 				AND graph_template_id = ?',
-				array($host_id, $unused_graph_template['id']));
+				array($device_id, $unused_graph_template['id']));
 
 			if ($poller_id > 1) {
 				if (remote_poller_up($poller_id)) {
-					if (($rcnn_id = poller_push_to_remote_db_connect($host_id)) !== false) {
+					if (($rcnn_id = poller_push_to_remote_db_connect($device_id)) !== false) {
 						db_execute_prepared('DELETE
 							FROM host_graph
 							WHERE host_id = ?
 							AND graph_template_id = ?',
-							array($host_id, $unused_graph_template['id']), true, $rcnn_id);
+							array($device_id, $unused_graph_template['id']), true, $rcnn_id);
 					} elseif (!isset($raised[$poller_id])) {
 						raise_message('poller_down_' . $poller_id, __('Remote Poller %s is Down, you will need to perform a FullSync once it is up again', $poller_id), MESSAGE_LEVEL_WARN);
 						$raised[$poller_id] = true;
 					}
-				} elseif (!$isset(raised[$poller_id])) {
+				} elseif (!$isset($raised[$poller_id])) {
 					raise_message('poller_down_' . $poller_id, __('Remote Poller %s is Down, you will need to perform a FullSync once it is up again', $poller_id), MESSAGE_LEVEL_WARN);
 					$raised[$poller_id] = true;
 				}
@@ -1220,23 +1357,28 @@ function api_device_update_host_template($host_id, $host_template_id) {
 	}
 }
 
-/* api_device_template_sync_template - updates the device template mapping for all devices mapped to a template
-   @arg $device_template - the device template to syncronize
-   @arg $host_ids - an array of host_ids or a string with a single host_id
-   @arg $down_devices - also update mapping of down devices */
-function api_device_template_sync_template($device_template, $host_ids = '', $down_devices = false) {
+/**
+ * api_device_template_sync_template - updates the device template mapping for all devices mapped to a template
+ *
+ * @param  (int)       The device template to syncronize
+ * @param  (int|array) An array of device_ids or a string with a single device_id
+ * @param  (bool)      Also update mapping of down devices
+ *
+ * @return (void)
+ */
+function api_device_template_sync_template($device_template, $device_ids = '', $down_devices = false) {
 	if ($down_devices == true) {
 		$status_where = '';
 	} else {
 		$status_where = ' AND status IN(3,2)';
 	}
 
-	if (is_array($host_ids)) {
-		$host_ids = implode(',', $host_ids);
+	if (is_array($device_ids)) {
+		$device_ids = implode(',', $device_ids);
 	}
 
-	if ($host_ids != '') {
-		$status_where .= ' AND host.id IN(' . $host_ids . ')';
+	if ($device_ids != '') {
+		$status_where .= ' AND host.id IN(' . $device_ids . ')';
 	}
 
 	$devices = array_rekey(
@@ -1255,6 +1397,15 @@ function api_device_template_sync_template($device_template, $host_ids = '', $do
 	}
 }
 
+/**
+ * api_device_ping_device - given a device id and optional indicator of where the ping request
+ *   came from, ping the device.  The ping results are echoed to standard output for the browser
+ *
+ * @param (int)  The device id in question
+ * @param (bool) Whether the source of the ping request is coming from a remote data collector.
+ *
+ * @return (void)
+ */
 function api_device_ping_device($device_id, $from_remote = false) {
 	global $config, $snmp_error;
 
