@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2021 The Cacti Group                                 |
+ | Copyright (C) 2004-2022 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -22,48 +22,129 @@
  +-------------------------------------------------------------------------+
 */
 
-/* get_rrdfile_names - this routine returns all of the RRDfiles know to Cacti
-     so as to be processed when performin the Daily, Weekly, Monthly and Yearly
-     average and peak calculations.
-   @returns - (mixed) The RRDfile names */
-function get_rrdfile_names() {
-	return db_fetch_assoc('SELECT data_template_data.local_data_id, data_source_path
-		FROM data_template_data
-		LEFT JOIN poller_item
-		ON poller_item.local_data_id = data_template_data.local_data_id
-		WHERE poller_item.local_data_id IS NOT NULL
-		AND data_template_data.local_data_id != 0');
+/**
+ * get_rrdfile_names - this routine returns all of the RRDfiles know to Cacti
+ *   so as to be processed when performin the Daily, Weekly, Monthly and Yearly
+ *   average and peak calculations.
+ *
+ * @param $thread_id   - (int) The thread to process
+ * @param $max_threads - (int) The maximum number of threads
+ *
+ * @return - (mixed) The RRDfile names
+ */
+function get_rrdfile_names($thread_id = 1, $max_threads = 1) {
+	static $newrows = array();
+
+	if ($max_threads == 1) {
+		return db_fetch_assoc('SELECT dtd.local_data_id, data_source_path, rrd_num AS dsses
+			FROM data_template_data AS dtd
+			LEFT JOIN poller_item AS pi
+			ON pi.local_data_id = dtd.local_data_id
+			WHERE pi.local_data_id IS NOT NULL
+			AND data_source_path != ""
+			AND dtd.local_data_id != 0');
+	} elseif (sizeof($newrows)) {
+		return $newrows[$thread_id];
+	} else {
+		$items_total = db_fetch_cell('SELECT COUNT(*)
+			FROM data_template_data AS dtd
+			LEFT JOIN poller_item AS pi
+			ON pi.local_data_id = dtd.local_data_id
+			WHERE pi.local_data_id IS NOT NULL
+			AND data_source_path != ""
+			AND dtd.local_data_id != 0');
+
+		$dsses_total = db_fetch_cell('SELECT SUM(rrd_num)
+			FROM data_template_data AS dtd
+			LEFT JOIN poller_item AS pi
+			ON pi.local_data_id = dtd.local_data_id
+			WHERE pi.local_data_id IS NOT NULL
+			AND data_source_path != ""
+			AND dtd.local_data_id != 0');
+
+		$split_size = ceil($dsses_total / $max_threads);
+
+		$rows = db_fetch_assoc('SELECT dtd.local_data_id, data_source_path, rrd_num AS dsses
+			FROM data_template_data AS dtd
+			LEFT JOIN poller_item AS pi
+			ON pi.local_data_id = dtd.local_data_id
+			WHERE pi.local_data_id IS NOT NULL
+			AND data_source_path != ""
+			AND dtd.local_data_id != 0');
+
+		$thread  = 1;
+		$dsses   = 0;
+		$newrows = array();
+
+		foreach($rows as $row) {
+			$dsses += $row['dsses'];
+
+			if ($dsses > $split_size && $thread < $max_threads) {
+				$thread++;
+				$dsses = 0;
+			}
+
+			$newrows[$thread][] = $row;
+		}
+
+		if (isset($newrows[$thread_id])) {
+			return $newrows[$thread_id];
+		} else {
+			return array();
+		}
+	}
 }
 
-/* dsstats_debug - this simple routine prints a standard message to the console
-     when running in debug mode.
-   @returns - NULL */
+/**
+ * dsstats_debug - this simple routine prints a standard message to the console
+ *   when running in debug mode.
+ *
+ * @param $message - (string) The message to display
+ *
+ * @return - NULL
+ */
 function dsstats_debug($message) {
 	global $debug;
 
 	if ($debug) {
-		print 'DSSTATS: ' . $message . "\n";
+		print 'DSSTATS: ' . $message . PHP_EOL;
 	}
 }
 
-/* dsstats_get_and_store_ds_avgpeak_values - this routine is a generic routine that takes an time interval as an
-     input parameter and then, though additional function calls, reads the RRDfiles for the correct information
-     and stores that information into the various database tables.
-   @arg $interval - (string) either 'daily', 'weekly', 'monthly', or 'yearly'
-   @returns - NULL */
-function dsstats_get_and_store_ds_avgpeak_values($interval) {
-	global $config;
-	global $user_time, $system_time, $real_time;
-	global $total_user_time, $total_system_time, $total_real_time;
+/**
+ * dsstats_get_and_store_ds_avgpeak_values - this routine is a generic routine that takes an time interval as an
+ *   input parameter and then, though additional function calls, reads the RRDfiles for the correct information
+ *   and stores that information into the various database tables.
+ *
+ * @param $interval  - (string) either 'daily', 'weekly', 'monthly', or 'yearly'
+ * @param $thread_id - (int) the dsstats parallel thread id
+ *
+ * @return - NULL
+ */
+function dsstats_get_and_store_ds_avgpeak_values($interval, $thread_id = 1) {
+	global $config, $type;
 
-	$rrdfiles = get_rrdfile_names();
-	$stats    = array();
+	global $total_user, $total_system, $total_real, $total_dsses;
+	global $user_time, $system_time, $real_time, $rrd_files;
 
 	$user_time   = 0;
 	$system_time = 0;
 	$real_time   = 0;
+	$dsses       = 0;
 
-	$use_proxy = (read_config_option('storage_location') ? true : false);
+	dsstats_debug(sprintf('Processing %s for Thread %s', $interval, $thread_id));
+
+	$max_threads = read_config_option('dsstats_parallel');
+	if (empty($max_threads)) {
+		$max_threads = 1;
+		set_config_option('dsstats_parallel', '1');
+	}
+
+	$rrdfiles   = get_rrdfile_names($thread_id, $max_threads);
+	$stats      = array();
+	$rrd_files += cacti_sizeof($rrdfiles);
+
+	$use_proxy  = (read_config_option('storage_location') ? true : false);
 
 	/* open a pipe to rrdtool for writing and reading */
 	if ($use_proxy) {
@@ -74,12 +155,10 @@ function dsstats_get_and_store_ds_avgpeak_values($interval) {
 		$pipes   = $process_pipes[1];
 	}
 
-	$system_time = 0;
-	$user_time   = 0;
-	$real_time   = 0;
-
 	if (cacti_sizeof($rrdfiles)) {
 		foreach ($rrdfiles as $file) {
+			$dsses += $file['dsses'];
+
 			if ($file['data_source_path'] != '') {
 				$rrdfile = str_replace('<path_rra>', $config['rra_path'], $file['data_source_path']);
 
@@ -101,20 +180,33 @@ function dsstats_get_and_store_ds_avgpeak_values($interval) {
 		dsstats_rrdtool_close($process);
 	}
 
-	$total_system_time += $system_time;
-	$total_user_time   += $user_time;
-	$total_real_time   += $real_time;
-
 	dsstats_write_buffer($stats, $interval);
+
+	if (!empty($type)) {
+		$total_user   += $user_time;
+		$total_system += $system_time;
+		$total_real   += $real_time;
+		$total_dsses  += $dsses;
+
+		set_config_option('dsstats_rrd_system_'  . $type . '_' . $thread_id, $total_system);
+		set_config_option('dsstats_rrd_user_'    . $type . '_' . $thread_id, $total_user);
+		set_config_option('dsstats_rrd_real_'    . $type . '_' . $thread_id, $total_real);
+		set_config_option('dsstats_total_rrds_'  . $type . '_' . $thread_id, $rrd_files);
+		set_config_option('dsstats_total_dsses_' . $type . '_' . $thread_id, $total_dsses);
+	}
 }
 
-/* dsstats_write_buffer - this routine provide bulk database insert services to the various tables that store
-     the average and peak information for Data Sources.
-   @arg $stats_array - (mixed) A multi dimensional array keyed by the local_data_id that contains both
-     the average and max values for each internal RRDfile Data Source.
-   @arg $interval - (string) 'daily', 'weekly', 'monthly', and 'yearly'.  Used for determining the table to
-     update during the dumping of the buffer.
-   @returns - NULL */
+/**
+ * dsstats_write_buffer - this routine provide bulk database insert services to the various tables that store
+ *   the average and peak information for Data Sources.
+ *
+ * @param $stats_array - (mixed) A multi dimensional array keyed by the local_data_id that contains both
+ *   the average and max values for each internal RRDfile Data Source.
+ * @param $interval    - (string) 'daily', 'weekly', 'monthly', and 'yearly'.  Used for determining the table to
+ *   update during the dumping of the buffer.
+ *
+ * @return - NULL
+ */
 function dsstats_write_buffer(&$stats_array, $interval) {
 	/* initialize some variables */
 	$sql_prefix = "INSERT INTO data_source_stats_$interval (local_data_id, rrd_name, average, peak) VALUES";
@@ -158,20 +250,27 @@ function dsstats_write_buffer(&$stats_array, $interval) {
 	}
 }
 
-/* dsstats_obtain_data_source_avgpeak_values - this routine, given the rrdfile name, interval and RRDtool process
-     pipes, will obtain the average a peak values from the RRDfile.  It does this in two steps:
+/**
+ * dsstats_obtain_data_source_avgpeak_values - this routine, given the rrdfile name, interval and RRDtool process
+ *   pipes, will obtain the average a peak values from the RRDfile.  It does this in two steps:
+ *
+ *   1) It first reads the RRDfile's information header to obtain all of the internal data source names,
+ *   poller interval and consolidation functions.
+ *   2) Based upon the available consolidation functions, it then grabs either AVERAGE, and MAX, or just AVERAGE
+ *      in the case where the MAX consolidation function is not included in the RRDfile, and then proceeds to
+ *      gather data from the RRDfile for the time period in question.  It allows RRDtool to select the RRA to
+ *      use by simply limiting the number of rows to be returned to the default.
 
-     1) It first reads the RRDfile's information header to obtain all of the internal data source names,
-     poller interval and consolidation functions.
-     2) Based upon the available consolidation functions, it then grabs either AVERAGE, and MAX, or just AVERAGE
-        in the case where the MAX consolidation function is not included in the RRDfile, and then proceeds to
-        gather data from the RRDfile for the time period in question.  It allows RRDtool to select the RRA to
-        use by simply limiting the number of rows to be returned to the default.
-
-     Once it has all of the information from the RRDfile.  It then decomposes the resulting XML file to its
-     components and then calculates the AVERAGE and MAX values from that data and returns an array to the calling
-     function for storage into the respective database table.
-   @returns - (mixed) An array of AVERAGE, and MAX values in an RRDfile by Data Source name */
+ *   Once it has all of the information from the RRDfile.  It then decomposes the resulting XML file to its
+ *   components and then calculates the AVERAGE and MAX values from that data and returns an array to the calling
+ *   function for storage into the respective database table.
+ *
+ * @param $rrdfile  - (string) The rrdfile to process
+ * @param $interval - (string) The interval type to process
+ * @param $pipes    - (resource) Pipes to the background RRDtool process
+ *
+ * @return - (mixed) An array of AVERAGE, and MAX values in an RRDfile by Data Source name
+ */
 function dsstats_obtain_data_source_avgpeak_values($rrdfile, $interval, &$pipes) {
 	global $config, $user_time, $system_time, $real_time;
 
@@ -207,18 +306,21 @@ function dsstats_obtain_data_source_avgpeak_values($rrdfile, $interval, &$pipes)
 			if (cacti_sizeof($info_array)) {
 				foreach ($info_array as $line) {
 					if (substr_count($line, 'ds[')) {
-						$parts = explode(']', $line);
+						$parts  = explode(']', $line);
 						$parts2 = explode('[', $parts[0]);
+
 						$dsnames[trim($parts2[1])] = 1;
-					} else if (substr_count($line, '.cf')) {
+					} elseif (substr_count($line, '.cf')) {
 						$parts = explode('=', $line);
+
 						if (substr_count($parts[1], 'AVERAGE')) {
 							$average = true;
 						} elseif (substr_count($parts[1], 'MAX')) {
 							$max = true;
 						}
-					} else if (substr_count($line, 'step')) {
+					} elseif (substr_count($line, 'step')) {
 						$parts = explode('=', $line);
+
 						$poller_interval = trim($parts[1]);
 					}
 				}
@@ -292,8 +394,8 @@ function dsstats_obtain_data_source_avgpeak_values($rrdfile, $interval, &$pipes)
 
 			/* initialize the array of return values */
 			foreach($dsnames as $dsname => $present) {
-				$dsvalues[$dsname]['AVG']    = 0;
-				$dsvalues[$dsname]['MAX']    = 0;
+				$dsvalues[$dsname]['AVG'] = 0;
+				$dsvalues[$dsname]['MAX'] = 0;
 
 				if ($average) {
 					$position[] = array($dsname => 'AVG');
@@ -311,7 +413,9 @@ function dsstats_obtain_data_source_avgpeak_values($rrdfile, $interval, &$pipes)
 
 				if (cacti_sizeof($xport_array)) {
 					foreach($xport_array as $index => $line) {
-						if ($line == '') continue;
+						if ($line == '') {
+							continue;
+						}
 
 						if ($index > 0) {
 							// Catch the last line
@@ -322,6 +426,7 @@ function dsstats_obtain_data_source_avgpeak_values($rrdfile, $interval, &$pipes)
 
 								foreach($parts as $line) {
 									$sparts = explode(':', $line);
+
 									switch($sparts[0]) {
 										case 'u':
 											$user_time = $sparts[1];
@@ -356,21 +461,69 @@ function dsstats_obtain_data_source_avgpeak_values($rrdfile, $interval, &$pipes)
 	}
 }
 
-/* log_dsstats_statistics - provides generic timing message to both the Cacti log and the settings
-     table so that the statistcs can be graphed as well.
-   @arg $type - (string) the type of statistics to log, either 'HOURLY', 'DAILY' or 'MAJOR'.
-   @returns - null */
-function log_dsstats_statistics($type) {
-	global $start, $total_user_time, $total_system_time, $total_real_time;;
+/**
+ * dsstats_log_statistics - provides generic timing message to both the Cacti log and the settings
+ *   table so that the statistcs can be graphed as well.
+ *
+ * @param $type - (string) the type of statistics to log, either 'HOURLY', 'DAILY', 'BOOST' or 'MAJOR'.
+ *
+ * @return - NULL
+ */
+function dsstats_log_statistics($type) {
+	global $start;
+
+	dsstats_debug($type);
+
+	if ($type == 'HOURLY') {
+		$sub_type = '';
+	} elseif ($type == 'MAJOR') {
+		$sub_type = 'dchild';
+	} elseif ($type == 'DAILY') {
+		$sub_type = 'child';
+	} elseif ($type == 'BOOST') {
+		$sub_type = 'bchild';
+	}
 
 	/* take time and log performance data */
 	$end = microtime(true);
 
-	$cacti_stats = sprintf('Time:%01.4f RRDUser:%01.4f RRDSystem:%01.4f RRDReal:%01.4f', round($end-$start,4), $total_user_time, $total_system_time, $total_real_time);
+	if ($sub_type != '') {
+		$rrd_user = db_fetch_cell_prepared("SELECT SUM(value)
+			FROM settings
+			WHERE name LIKE ?",
+			array('dsstats_rrd_user_%' . $sub_type . '%'));
 
-	$total_user_time   = 0;
-	$total_system_time = 0;
-	$total_real_time   = 0;
+		$rrd_system = db_fetch_cell_prepared("SELECT SUM(value)
+			FROM settings
+			WHERE name LIKE ?",
+			array('dsstats_rrd_system_%' . $sub_type . '%'));
+
+		$rrd_real = db_fetch_cell_prepared("SELECT SUM(value)
+			FROM settings
+			WHERE name LIKE ?",
+			array('dsstats_rrd_real_%' . $sub_type . '%'));
+
+		$rrd_files = db_fetch_cell_prepared("SELECT SUM(value)
+			FROM settings
+			WHERE name LIKE ?",
+			array('dsstats_total_rrds_%' . $sub_type . '%'));
+
+		$dsses = db_fetch_cell_prepared("SELECT SUM(value)
+			FROM settings
+			WHERE name LIKE ?",
+			array('dsstats_total_dsses_%' . $sub_type . '%'));
+
+		$processes  = read_config_option('dsstats_parallel');
+
+		$cacti_stats = sprintf('Time:%01.2f Type:%s Threads:%s RRDfiles:%s DSSes:%s RRDUser:%01.2f RRDSystem:%01.2f RRDReal:%01.2f', $end - $start, $type, $processes, $rrd_files, $dsses, $rrd_user, $rrd_system, $rrd_real);
+
+		db_execute("DELETE FROM settings
+			WHERE name LIKE 'dsstats_rrd_%$sub_type%'
+			OR name LIKE 'dsstats_total_rrds_%$sub_type%'
+			OR name LIKE 'dsstats_total_dsses_%$sub_type%'");
+	} else {
+		$cacti_stats = sprintf('Time:%01.2f Type:%s', $end-$start, $type);
+	}
 
 	/* take time and log performance data */
 	$start = microtime(true);
@@ -379,18 +532,62 @@ function log_dsstats_statistics($type) {
 	set_config_option('stats_dsstats_' . $type, $cacti_stats);
 
 	/* log to the logfile */
-	cacti_log('DSSTATS STATS: Type:' . $type . ', ' . $cacti_stats , true, 'SYSTEM');
+	cacti_log('DSSTATS STATS: ' . $cacti_stats , true, 'SYSTEM');
 }
 
-/* dsstats_error_handler - this routine logs all PHP error transactions
-     to make sure they are properly logged.
-   @arg $errno - (int) The errornum reported by the system
-   @arg $errmsg - (string) The error message provides by the error
-   @arg $filename - (string) The filename that encountered the error
-   @arg $linenum - (int) The line number where the error occurred
-   @arg $vars - (mixed) The current state of PHP variables.
-   @returns - (bool) always returns true for some reason */
-function dsstats_error_handler($errno, $errmsg, $filename, $linenum, $vars) {
+/**
+ * dsstats_log_child_stats - logs dsstats child process information
+ *
+ * @param $type        - (string) The type of child, MAJOR, DAILY, BOOST
+ * @param $thread_id   - (int) The parallel thread id
+ * @param $total_time  - (int) The total time to collect date
+ *
+ * @return - NULL
+ */
+function dsstats_log_child_stats($type, $thread_id, $total_time) {
+	$rrd_user = db_fetch_cell_prepared("SELECT SUM(value)
+		FROM settings
+		WHERE name LIKE ?",
+		array('dsstats_rrd_user_%' . $type . '_' . $thread_id . '%'));
+
+	$rrd_system = db_fetch_cell_prepared("SELECT SUM(value)
+		FROM settings
+		WHERE name LIKE ?",
+		array('dsstats_rrd_system_%' . $type . '_' . $thread_id . '%'));
+
+	$rrd_real = db_fetch_cell_prepared("SELECT SUM(value)
+		FROM settings
+		WHERE name LIKE ?",
+		array('dsstats_rrd_real_%' . $type . '_' . $thread_id . '%'));
+
+	$rrd_files = db_fetch_cell_prepared("SELECT SUM(value)
+		FROM settings
+		WHERE name LIKE ?",
+		array('dsstats_total_rrds_%' . $type . '_' . $thread_id . '%'));
+
+	$dsses = db_fetch_cell_prepared("SELECT SUM(value)
+		FROM settings
+		WHERE name LIKE ?",
+		array('dsstats_total_dsses_%' . $type . '_' . $thread_id . '%'));
+
+	$cacti_stats = sprintf('Time:%01.2f Type:%s ProcessNumber:%s RRDfiles:%s DSSes:%s RRDUser:%01.2f RRDSystem:%01.2f RRDReal:%01.2f', $total_time, strtoupper($type), $thread_id, $rrd_files, $dsses, $rrd_user, $rrd_system, $rrd_real);
+
+	cacti_log('DSSTATS CHILD STATS: ' . $cacti_stats, true, 'SYSTEM');
+}
+
+/**
+ * dsstats_error_handler - this routine logs all PHP error transactions
+ *   to make sure they are properly logged.
+ *
+ * @param $errno    - (int) The errornum reported by the system
+ * @param $errmsg   - (string) The error message provides by the error
+ * @param $filename - (string) The filename that encountered the error
+ * @param $linenum  - (int) The line number where the error occurred
+ * @param $vars     - (mixed) The current state of PHP variables.
+ *
+ * @returns - (bool) always returns true for some reason
+ */
+function dsstats_error_handler($errno, $errmsg, $filename, $linenum, $vars = []) {
 	if (read_config_option('log_verbosity') >= POLLER_VERBOSITY_DEBUG) {
 		/* define all error types */
 		$errortype = array(
@@ -428,33 +625,37 @@ function dsstats_error_handler($errno, $errmsg, $filename, $linenum, $vars) {
 	return;
 }
 
-/* dsstats_poller_output - this routine runs in parallel with the cacti poller and
-     populates the last and cache tables.  On larger systems, it should be noted that
-     the memory overhead for the global arrays, $ds_types, $ds_last, $ds_steps, $ds_multi
-     could be serval hundred megabytes.  So, this should be kept in mind when running the
-     sizing your system.
-
-     The routine basically loads those 4 structures into memory, and then uses them to
-     determine what should be stored in both the Cache and the Last tables.  The 4 structures
-     contain the following information:
-
-     $ds_types - The type of data source, keyed by the local_data_id and the rrd_name stored inside
-                 of the RRDfile.
-     $ds_last  - For the COUNTER, and DERIVE DS types, the last measured and stored value.
-     $ds_steps - Records the poller interval for every Data Source so that rates can be stored.
-     $ds_multi - For Multi Part responses, stores the mapping of the Data Input Fields to the
-                 Internal RRDfile DS names.
-
-     The routine loops through all poller output items and makes decisions relative to the output
-     that should be stored into the two tables, and then bulk inserts that information once
-     all poller items have been processed.
-
-     The pupose for loading then entire structures into memory at one time is to reduce the latency
-     related to multiple database calls.  The author believed that PHP's array hashing algorythms
-     would be as fast, if not faster, than MySQL, when considering the transaction overhead and therefore
-     chose this method.
-
-   @arg $rrd_update_array - (mixed) The output from the poller output table to be processed by dsstats */
+/**
+ * dsstats_poller_output - this routine runs in parallel with the cacti poller and
+ *   populates the last and cache tables.  On larger systems, it should be noted that
+ *   the memory overhead for the global arrays, $ds_types, $ds_last, $ds_steps, $ds_multi
+ *   could be serval hundred megabytes.  So, this should be kept in mind when running the
+ *   sizing your system.
+ *
+ *   The routine basically loads those 4 structures into memory, and then uses them to
+ *   determine what should be stored in both the Cache and the Last tables.  The 4 structures
+ *   contain the following information:
+ *
+ *   $ds_types - The type of data source, keyed by the local_data_id and the rrd_name stored inside
+ *               of the RRDfile.
+ *   $ds_last  - For the COUNTER, and DERIVE DS types, the last measured and stored value.
+ *   $ds_steps - Records the poller interval for every Data Source so that rates can be stored.
+ *   $ds_multi - For Multi Part responses, stores the mapping of the Data Input Fields to the
+ *               Internal RRDfile DS names.
+ *
+ *   The routine loops through all poller output items and makes decisions relative to the output
+ *   that should be stored into the two tables, and then bulk inserts that information once
+ *   all poller items have been processed.
+ *
+ *   The pupose for loading then entire structures into memory at one time is to reduce the latency
+ *   related to multiple database calls.  The author believed that PHP's array hashing algorythms
+ *   would be as fast, if not faster, than MySQL, when considering the transaction overhead and therefore
+ *   chose this method.
+ *
+ * @param $rrd_update_array - (mixed) The output from the poller output table to be processed by dsstats
+ *
+ * @return - NULL
+ */
 function dsstats_poller_output(&$rrd_update_array) {
 	global $config, $ds_types, $ds_last, $ds_steps, $ds_multi;
 
@@ -485,22 +686,22 @@ function dsstats_poller_output(&$rrd_update_array) {
 
 			/* determine the keyvalue pairs to decide on how to store data */
 			$ds_types = array_rekey(
-				db_fetch_assoc('SELECT DISTINCT data_source_name, data_source_type_id, rrd_step
-					FROM data_template_rrd
-					INNER JOIN data_template_data
-					ON data_template_data.local_data_id=data_template_rrd.local_data_id
-					WHERE data_template_data.local_data_id>0'),
-				'data_source_name', array('data_source_type_id', 'rrd_step')
+				db_fetch_assoc('SELECT DISTINCT data_source_name, data_source_type_id, rrd_step, rrd_maximum
+					FROM data_template_rrd AS dtr
+					INNER JOIN data_template_data AS dtd
+					ON dtd.local_data_id = dtr.local_data_id
+					WHERE dtd.local_data_id > 0'),
+				'data_source_name', array('data_source_type_id', 'rrd_step', 'rrd_maximum')
 			);
 
 			/* make the association between the multi-part name value pairs and the RRDfile internal
 			 * data source names.
 			 */
 			$ds_multi = array_rekey(db_fetch_assoc('SELECT DISTINCT data_name, data_source_name
-				FROM data_template_rrd
-				INNER JOIN data_input_fields
-				ON data_input_fields.id=data_template_rrd.data_input_field_id
-				WHERE data_template_rrd.data_input_field_id!=0'), 'data_name', 'data_source_name');
+				FROM data_template_rrd AS dtr
+				INNER JOIN data_input_fields AS dif
+				ON dif.id = dtr.data_input_field_id
+				WHERE dtr.data_input_field_id != 0'), 'data_name', 'data_source_name');
 
 			/* required for updating tables */
 			$cache_i      = 1;
@@ -558,7 +759,8 @@ function dsstats_poller_output(&$rrd_update_array) {
 								$ds_last = db_fetch_cell_prepared('SELECT SQL_NO_CACHE `value`
 									FROM data_source_stats_hourly_last
 									WHERE local_data_id = ?
-									AND rrd_name = ?', array($result['local_data_id'], $result['rrd_name']));
+									AND rrd_name = ?',
+									array($result['local_data_id'], $result['rrd_name']));
 
 								if ($ds_last == '' || $ds_last == 'NULL') {
 									$currentval = 'NULL';
@@ -568,12 +770,18 @@ function dsstats_poller_output(&$rrd_update_array) {
 									/* everything is normal */
 									$currentval = $result['output'] - $ds_last;
 								} else {
+									$max_value = $ds_types[$result['rrd_name']]['rrd_maximum'];
+
 									/* possible overflow, see if its 32bit or 64bit */
 									if ($ds_last > 4294967295) {
 										$currentval = (18446744073709551615 - $ds_last) + $result['output'];
 									} else {
 										$currentval = (4294967295 - $ds_last) + $result['output'];
 									}
+
+									if ($max_value != 'U' && $currentval > $max_value) {
+										$currentval = 'NULL';
+									}
 								}
 
 								if ($currentval != 'NULL') {
@@ -731,10 +939,13 @@ function dsstats_poller_output(&$rrd_update_array) {
 	restore_error_handler();
 }
 
-/* dsstats_boost_bottom - this routine accomodates mass updates after the boost process
-     has completed.  The use of boost will require boost version 2.5 or above.  The idea
-     if that daily averages will be updated on the boost cycle.
-   @returns - NULL */
+/**
+ * dsstats_boost_bottom - this routine accomodates mass updates after the boost process
+ *   has completed.  The use of boost will require boost version 2.5 or above.  The idea
+ *   if that daily averages will be updated on the boost cycle.
+ *
+ * @return - NULL
+ */
 function dsstats_boost_bottom() {
 	global $config;
 
@@ -744,23 +955,39 @@ function dsstats_boost_bottom() {
 		/* run the daily stats. log to database to prevent secondary runs */
 		set_config_option('dsstats_last_daily_run_time', date('Y-m-d G:i:s', time()));
 
+		/* run the daily stats */
+		dsstats_launch_children('bmaster');
+
+		/* Wait for all processes to continue */
+		while ($running = dsstats_processes_running('bmaster')) {
+			dsstats_debug(sprintf('%s Processes Running, Sleeping for 2 seconds.', $running));
+			sleep(2);
+		}
+
 		dsstats_get_and_store_ds_avgpeak_values('daily');
 
-		log_dsstats_statistics('DAILY');
+		dsstats_log_statistics('DAILY');
 	}
 }
 
-/* dsstats_memory_limit - this routine increases/decreases the memory available for the script
-     It is divided into two functions as the main dsstats poller calls this function directly
-     as opposed to the call during the processing of poller output in the main cacti poller.
-   @returns - NULL */
+/**
+ * dsstats_memory_limit - this routine increases/decreases the memory available for the script
+ *   It is divided into two functions as the main dsstats poller calls this function directly
+ *   as opposed to the call during the processing of poller output in the main cacti poller.
+ *
+ * @return - NULL
+ */
 function dsstats_memory_limit() {
 	ini_set('memory_limit', read_config_option('dsstats_poller_mem_limit') . 'M');
 }
 
-/* dsstats_poller_bottom - this routine launches the main dsstats poller so that it might
-     calculate the Hourly, Daily, Weekly, Monthly, and Yearly averages.  It is forked independently
-     to the Cacti poller after all polling has finished. */
+/**
+ * dsstats_poller_bottom - this routine launches the main dsstats poller so that it might
+ *   calculate the Hourly, Daily, Weekly, Monthly, and Yearly averages.  It is forked independently
+ *   to the Cacti poller after all polling has finished.
+ *
+ * @return - NULL
+ */
 function dsstats_poller_bottom () {
 	global $config;
 
@@ -784,11 +1011,14 @@ function dsstats_poller_bottom () {
 	}
 }
 
-/* dsstats_rrdtool_init - this routine provides a bi-directional socket based connection to RRDtool.
-     it provides a high speed connection to rrdfile in the case where the traditional Cacti call does
-     not when performing fetch type calls.
-   @returns - (mixed) An array that includes both the process resource and the pipes to communicate
-     with RRDtool. */
+/**
+ * dsstats_rrdtool_init - this routine provides a bi-directional socket based connection to RRDtool.
+ *   it provides a high speed connection to rrdfile in the case where the traditional Cacti call does
+ *   not when performing fetch type calls.
+ *
+ * @return - (mixed) An array that includes both the process resource and the pipes to communicate
+ *   with RRDtool.
+ */
 function dsstats_rrdtool_init() {
 	global $config;
 
@@ -822,14 +1052,18 @@ function dsstats_rrdtool_init() {
 	return array($process, $pipes);
 }
 
-/* dsstats_rrdtool_execute - this routine passes commands to RRDtool and returns the information
-     back to DSStats.  It is important to note here that RRDtool needs to provide an either 'OK'
-     or 'ERROR' response accross the pipe as it does not provide EOF characters to key upon.
-     This may not be the best method and may be changed after I have a conversation with a few
-     developers.
-   @arg $command - (string) The rrdtool command to execute
-   @arg $pipes - (array) An array of stdin and stdout pipes to read and write data from
-   @returns - (string) The output from RRDtool */
+/**
+ * dsstats_rrdtool_execute - this routine passes commands to RRDtool and returns the information
+ *   back to DSStats.  It is important to note here that RRDtool needs to provide an either 'OK'
+ *   or 'ERROR' response accross the pipe as it does not provide EOF characters to key upon.
+ *   This may not be the best method and may be changed after I have a conversation with a few
+ *   developers.
+ *
+ * @param $command - (string) The rrdtool command to execute
+ * @param $pipes   - (array) An array of stdin and stdout pipes to read and write data from
+ *
+ * @returns - (string) The output from RRDtool
+ */
 function dsstats_rrdtool_execute($command, &$pipes) {
 	static $broken = false;
 
@@ -860,10 +1094,128 @@ function dsstats_rrdtool_execute($command, &$pipes) {
 	if (strlen($stdout)) return $stdout;
 }
 
-/* dsstats_rrdtool_close - this routine closes the RRDtool process thus also
-     closing the pipes.
-   @returns - NULL */
+/**
+ * dsstats_rrdtool_close - this routine closes the RRDtool process thus also
+ *   closing the pipes.
+ *
+ * @return - NULL
+ */
 function dsstats_rrdtool_close($process) {
 	proc_close($process);
 }
 
+/**
+ * dsstats_launch_children - this function will launch collector children based upon
+ *   the maximum number of threads and the process type
+ *
+ * @param $type - (string) The process type
+ *
+ * @return - NULL
+ */
+function dsstats_launch_children($type) {
+	global $config, $debug;
+
+	$processes = read_config_option('dsstats_parallel');
+
+	if (empty($processes)) {
+		$processes = 1;
+	}
+
+	$php_binary = read_config_option('path_php_binary');
+
+	dsstats_debug("About to launch $processes processes.");
+
+	$sub_type = dsstats_get_subtype($type);
+
+	for ($i = 1; $i <= $processes; $i++) {
+		dsstats_debug(sprintf('Launching DSStats Process Number %s for Type %s', $i, $type));
+
+		cacti_log(sprintf('NOTE: Launching DSStats Process Number %s for Type %s', $i, $type), false, 'BOOST', POLLER_VERBOSITY_MEDIUM);
+
+		exec_background($php_binary, $config['base_path'] . "/poller_dsstats.php --type=$sub_type --child=$i" . ($debug ? ' --debug':''));
+	}
+
+	sleep(2);
+}
+
+/**
+ * dsstats_get_subtype - this function determine the applicable
+ *   sub-type (child name) and return if based upon a type
+ *
+ * @param $type - (string) The process type
+ *
+ * @return - (string) The sub type
+ */
+function dsstats_get_subtype($type) {
+	switch($type) {
+		case 'master':
+			return 'child';
+			break;
+		case 'bmaster':
+			return 'bchild';
+			break;
+		case 'dmaster':
+			return 'dchild';
+			break;
+	}
+}
+
+/**
+ * dsstats_kill_running_processes - this function is part of an interrupt
+ *   handler to kill children processes when the parent is killed
+ *
+ * @return - NULL
+ */
+function dsstats_kill_running_processes() {
+	global $type;
+
+	if ($type == 'bmaster') {
+		$processes = db_fetch_assoc_prepared('SELECT *
+			FROM processes
+			WHERE tasktype = "dsstats"
+			AND taskname = "bchild"
+			AND pid != ?',
+			array(getmypid()));
+	} else {
+		$processes = db_fetch_assoc_prepared('SELECT *
+			FROM processes
+			WHERE tasktype = "dsstats"
+			AND taskname IN ("child", "dchild")
+			AND pid != ?',
+			array(getmypid()));
+	}
+
+	if (cacti_sizeof($processes)) {
+		foreach($processes as $p) {
+			cacti_log(sprintf('WARNING: Killing DSStats %s PID %d due to another due to signal or overrun.', ucfirst($p['taskname']), $p['pid']), false, 'BOOST');
+			posix_kill($p['pid'], SIGTERM);
+
+			unregister_process($p['tasktype'], $p['taskname'], $p['taskid'], $p['pid']);
+		}
+	}
+}
+
+/**
+ * dsstats_processes_running - given a type, determine the number
+ *   of sub-type or children that are currently running
+ *
+ * @param $type - (string) The process type
+ *
+ * @return - (int) The number of running processes
+ */
+function dsstats_processes_running($type) {
+	$sub_type = dsstats_get_subtype($type);
+
+	$running = db_fetch_cell_prepared('SELECT COUNT(*)
+		FROM processes
+		WHERE tasktype = "dsstats"
+		AND taskname = ?',
+		array($sub_type));
+
+	if ($running == 0) {
+		return 0;
+	}
+
+	return $running;
+}
+
