@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2019 The Cacti Group                                 |
+ | Copyright (C) 2004-2022 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -25,6 +25,9 @@
 function graph_template_to_xml($graph_template_id) {
 	global $struct_graph, $fields_graph_template_input_edit, $struct_graph_item, $export_errors;
 
+	// Remote caching item
+	unset($struct_graph_item['data_template_id']);
+
 	$hash['graph_template'] = get_hash_version('graph_template') . get_hash_graph_template($graph_template_id);
 	$xml_text = '';
 
@@ -36,14 +39,15 @@ function graph_template_to_xml($graph_template_id) {
 	$graph_template_graph = db_fetch_row_prepared('SELECT *
 		FROM graph_templates_graph
 		WHERE graph_template_id = ?
-		AND local_graph_id=0
+		AND local_graph_id = 0
 		ORDER BY id',
 		array($graph_template_id));
 
 	$graph_template_items = db_fetch_assoc_prepared('SELECT *
 		FROM graph_templates_item
 		WHERE graph_template_id = ?
-		AND local_graph_id=0
+		AND local_graph_id = 0
+		AND hash != ""
 		ORDER BY sequence',
 		array($graph_template_id));
 
@@ -189,7 +193,8 @@ function data_template_to_xml($data_template_id) {
 	$data_template_rrd = db_fetch_assoc_prepared('SELECT *
 		FROM data_template_rrd
 		WHERE data_template_id = ?
-		AND local_data_id=0
+		AND local_data_id = 0
+		AND hash != ""
 		ORDER BY id',
 		array($data_template_id));
 
@@ -332,7 +337,7 @@ function data_input_method_to_xml($data_input_id) {
 
 	$xml_text .= "\t<fields>\n";
 
-	if (cacti_sizeof($data_input_fields) > 0) {
+	if (cacti_sizeof($data_input_fields)) {
 		foreach ($data_input_fields as $item) {
 			$hash['data_input_field'] = get_hash_version('data_input_field') . get_hash_data_input($item['id'], 'data_input_field');
 
@@ -342,7 +347,16 @@ function data_input_method_to_xml($data_input_id) {
 				if (($field_name == 'input_output') && (!empty($item[$field_name]))) {
 					$xml_text .= "\t\t\t<$field_name>" . xml_character_encode($item[$field_name]) . "</$field_name>\n";
 				} else {
-					if (($field_array['method'] != 'hidden_zero') && ($field_array['method'] != 'hidden')) {
+					$method = $field_array['method'];
+
+					if ($method != 'hidden_zero' && $method != 'hidden' && $method != 'spacer') {
+						// Work around renaming of field in form
+						// To get around JavaScript form issue
+						// In data_input.php
+						if ($field_name == 'fname') {
+							$field_name = 'name';
+						}
+
 						$xml_text .= "\t\t\t<$field_name>" . xml_character_encode($item[$field_name]) . "</$field_name>\n";
 					}
 				}
@@ -782,9 +796,9 @@ function data_query_to_xml($data_query_id) {
 				foreach ($snmp_query_graph_rrd as $item2) {
 					$xml_text .= "\t\t\t\t<item_" . str_pad(strval($i), 3, '0', STR_PAD_LEFT) . ">\n";
 
-					$xml_text .= "\t\t\t\t\t<snmp_field_name>" . $item2{'snmp_field_name'} . "</snmp_field_name>\n";
-					$xml_text .= "\t\t\t\t\t<data_template_id>hash_" . get_hash_version('data_template') . get_hash_data_template($item2{'data_template_id'}) . "</data_template_id>\n";
-					$xml_text .= "\t\t\t\t\t<data_template_rrd_id>hash_" . get_hash_version('data_template_item') . get_hash_data_template($item2{'data_template_rrd_id'}, 'data_template_item') . "</data_template_rrd_id>\n";
+					$xml_text .= "\t\t\t\t\t<snmp_field_name>" . $item2['snmp_field_name'] . "</snmp_field_name>\n";
+					$xml_text .= "\t\t\t\t\t<data_template_id>hash_" . get_hash_version('data_template') . get_hash_data_template($item2['data_template_id']) . "</data_template_id>\n";
+					$xml_text .= "\t\t\t\t\t<data_template_rrd_id>hash_" . get_hash_version('data_template_item') . get_hash_data_template($item2['data_template_rrd_id'], 'data_template_item') . "</data_template_rrd_id>\n";
 
 					$xml_text .= "\t\t\t\t</item_" . str_pad(strval($i), 3, '0', STR_PAD_LEFT) . ">\n";
 
@@ -805,9 +819,9 @@ function data_query_to_xml($data_query_id) {
 
 					$xml_text .= "\t\t\t\t<hash_" . $hash['data_query_sv_graph'] . ">\n";
 
-					$xml_text .= "\t\t\t\t\t<field_name>" . xml_character_encode($item2{'field_name'}) . "</field_name>\n";
-					$xml_text .= "\t\t\t\t\t<sequence>" . $item2{'sequence'} . "</sequence>\n";
-					$xml_text .= "\t\t\t\t\t<text>" . xml_character_encode($item2{'text'}) . "</text>\n";
+					$xml_text .= "\t\t\t\t\t<field_name>" . xml_character_encode($item2['field_name']) . "</field_name>\n";
+					$xml_text .= "\t\t\t\t\t<sequence>" . $item2['sequence'] . "</sequence>\n";
+					$xml_text .= "\t\t\t\t\t<text>" . xml_character_encode($item2['text']) . "</text>\n";
 
 					$xml_text .= "\t\t\t\t</hash_" . $hash['data_query_sv_graph'] . ">\n";
 
@@ -828,10 +842,10 @@ function data_query_to_xml($data_query_id) {
 
 					$xml_text .= "\t\t\t\t<hash_" . $hash['data_query_sv_data_source'] . ">\n";
 
-					$xml_text .= "\t\t\t\t\t<field_name>" . xml_character_encode($item2{'field_name'}) . "</field_name>\n";
-					$xml_text .= "\t\t\t\t\t<data_template_id>hash_" . get_hash_version('data_template') . get_hash_data_template($item2{'data_template_id'}) . "</data_template_id>\n";
-					$xml_text .= "\t\t\t\t\t<sequence>" . $item2{'sequence'} . "</sequence>\n";
-					$xml_text .= "\t\t\t\t\t<text>" . xml_character_encode($item2{'text'}) . "</text>\n";
+					$xml_text .= "\t\t\t\t\t<field_name>" . xml_character_encode($item2['field_name']) . "</field_name>\n";
+					$xml_text .= "\t\t\t\t\t<data_template_id>hash_" . get_hash_version('data_template') . get_hash_data_template($item2['data_template_id']) . "</data_template_id>\n";
+					$xml_text .= "\t\t\t\t\t<sequence>" . $item2['sequence'] . "</sequence>\n";
+					$xml_text .= "\t\t\t\t\t<text>" . xml_character_encode($item2['text']) . "</text>\n";
 
 					$xml_text .= "\t\t\t\t</hash_" . $hash['data_query_sv_data_source'] . ">\n";
 
@@ -866,9 +880,9 @@ function resolve_dependencies($type, $id, $dep_array) {
 		$graph_template_items = db_fetch_assoc_prepared('SELECT
 			data_template_rrd.data_template_id
 			FROM (graph_templates_item,data_template_rrd)
-			WHERE graph_templates_item.task_item_id=data_template_rrd.id
+			WHERE graph_templates_item.task_item_id = data_template_rrd.id
 			AND graph_templates_item.graph_template_id = ?
-			AND graph_templates_item.local_graph_id=0
+			AND graph_templates_item.local_graph_id = 0
 			AND graph_templates_item.task_item_id > 0
 			GROUP BY data_template_rrd.data_template_id', array($id));
 
@@ -884,7 +898,7 @@ function resolve_dependencies($type, $id, $dep_array) {
 		$cdef_items = db_fetch_assoc_prepared('SELECT cdef_id
 			FROM graph_templates_item
 			WHERE graph_template_id = ?
-			AND local_graph_id=0
+			AND local_graph_id = 0
 			AND cdef_id > 0
 			GROUP BY cdef_id', array($id));
 
@@ -932,7 +946,7 @@ function resolve_dependencies($type, $id, $dep_array) {
 		$vdef_items = db_fetch_assoc_prepared('SELECT vdef_id
 			FROM graph_templates_item
 			WHERE graph_template_id = ?
-			AND local_graph_id=0
+			AND local_graph_id = 0
 			AND vdef_id > 0
 			GROUP BY vdef_id',
 			array($id));
@@ -949,7 +963,7 @@ function resolve_dependencies($type, $id, $dep_array) {
 		$graph_template_items = db_fetch_assoc_prepared('SELECT gprint_id
 			FROM graph_templates_item
 			WHERE graph_template_id = ?
-			AND local_graph_id=0
+			AND local_graph_id = 0
 			AND gprint_id > 0
 			GROUP BY gprint_id',
 			array($id));
@@ -968,7 +982,7 @@ function resolve_dependencies($type, $id, $dep_array) {
 		$item = db_fetch_row_prepared('SELECT data_input_id
 			FROM data_template_data
 			WHERE data_template_id = ?
-			AND local_data_id=0
+			AND local_data_id = 0
 			AND data_input_id > 0',
 			array($id));
 
@@ -980,7 +994,7 @@ function resolve_dependencies($type, $id, $dep_array) {
 		$profiles = db_fetch_assoc_prepared('SELECT DISTINCT data_source_profile_id
 			FROM data_template_data
 			WHERE data_template_id = ?
-			AND local_data_id=0',
+			AND local_data_id = 0',
 			array($id));
 
 		if (cacti_sizeof($profiles)) {
