@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2021 The Cacti Group                                 |
+ | Copyright (C) 2004-2022 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -84,7 +84,22 @@ function cacti_ldap_auth($username, $password = '', $dn = '', $host = '', $port
 	if (!empty($group_attrib))      $ldap->group_attrib      = $group_attrib;
 	if (!empty($group_member_type)) $ldap->group_member_type = $group_member_type;
 
-	return $ldap->Authenticate();
+	/* If the server list is a space delimited set of servers
+	 * process each server until you get a bind, or fail
+	 */
+	$ldap_servers = preg_split('/\s+/', $ldap->host);
+
+	foreach($ldap_servers as $ldap_server) {
+		$ldap->host = $ldap_server;
+
+		$response = $ldap->Authenticate();
+
+		if ($response['error_num'] == 0) {
+			return $response;
+		}
+	}
+
+	return $response;
 }
 
 /* cacti_ldap_search_dn
@@ -149,7 +164,22 @@ function cacti_ldap_search_dn($username, $dn = '', $host = '', $port = '', $port
 	if (!empty($specific_dn))       $ldap->specific_dn       = $specific_dn;
 	if (!empty($specific_password)) $ldap->specific_password = $specific_password;
 
-	return $ldap->Search();
+	/* If the server list is a space delimited set of servers
+	 * process each server until you get a bind, or fail
+	 */
+	$ldap_servers = preg_split('/\s+/', $ldap->host);
+
+	foreach($ldap_servers as $ldap_server) {
+		$ldap->host = $ldap_server;
+
+		$response = $ldap->Search();
+
+		if ($response['error_num'] == 0) {
+			return $response;
+		}
+	}
+
+	return $response;
 }
 
 /* cacti_ldap_search_cn
@@ -235,9 +265,10 @@ abstract class LdapError {
 	const SearchFoundNoUser     = 14;
 	const SearchFoundNoUserDN   = 15;
 	const UndefinedDnOrPassword = 16;
+	const EmptyPassword         = 17;
 	const Disabled              = 99;
 
-	public static function GetErrorDetails($returnError, $ldapConn = null, $ldapError = 0) {
+	public static function GetErrorDetails($returnError, $ldapConn = null, $ldapServer = '', $ldapError = 0) {
 		$error_num  = $returnError;
 		$error_text = '';
 
@@ -264,39 +295,39 @@ abstract class LdapError {
 				break;
 
 			case LdapError::ProtocolErrorVersion:
-				$error_text = __('Protocol Error, Unable to set version (%s)', $ldapError);
+				$error_text = __('Protocol Error, Unable to set version (%s) on Server (%s)', $ldapError, $ldapServer);
 				break;
 
 			case LdapError::ProtocolErrorReferral:
-				$error_text = __('Protocol Error, Unable to set referrals option (%s)', $ldapError);
+				$error_text = __('Protocol Error, Unable to set referrals option (%s) on Server (%s)', $ldapError, $ldapServer);
 				break;
 
 			case LdapError::ProtocolErrorTls:
-				$error_text = __('Protocol Error, unable to start TLS communications (%s)', $ldapError);
+				$error_text = __('Protocol Error, unable to start TLS communications (%s) on Server (%s)', $ldapError, $ldapServer);
 				break;
 
 			case LdapError::ProtocolErrorGeneral:
-				$error_text = __('Protocol Error, General failure (%s)', $ldapError);
+				$error_text = __('Protocol Error, General failure (%s)', $ldapError, $ldapServer);
 				break;
 
 			case LdapError::ProtocolErrorBind:
-				$error_text = __('Protocol Error, Unable to bind, LDAP result: (%s)', $ldapError);
+				$error_text = __('Protocol Error, Unable to bind, LDAP result: (%s) on Server (%s)', $ldapError, $ldapServer);
 				break;
 
 			case LdapError::ConnectionUnavailable:
-				$error_text = __('Unable to Connect to Server');
+				$error_text = __('Unable to Connect to Server (%s)', $ldapServer);
 				break;
 
 			case LdapError::ConnectionTimeout:
-				$error_text =  __('Connection Timeout');
+				$error_text =  __('Connection Timeout to Server (%s)', $ldapServer);
 				break;
 
 			case LdapError::InsufficientAccess:
-				$error_text = __('Insufficient Access');
+				$error_text = __('Insufficient Access to Server (%s)', $ldapServer);
 				break;
 
 			case LdapError::SearchFoundNoGroup:
-				$error_text = __('Group DN could not be found to compare');
+				$error_text = __('Group DN could not be found to compare on Server (%s)', $ldapServer);
 				break;
 
 			case LdapError::SearchFoundMultiUser:
@@ -312,15 +343,19 @@ abstract class LdapError {
 				break;
 
 			case LdapError::MissingLdapObject:
-				$error_text = __('Unable to create LDAP connection object');
+				$error_text = __('Unable to create LDAP connection object to Server (%s)', $ldapServer);
 				break;
 
 			case LdapError::UndefinedDnOrPassword:
 				$error_text = __('Specific DN and Password required');
 				break;
 
+			case LdapError::EmptyPassword:
+				$error_text = __('Invalid Password provided.  Login failed.');
+				break;
+
 			default:
-				$error_text = __('Unexpected error %s (Ldap Error: %s)', $returnError, $ldapError);
+				$error_text = __('Unexpected error %s (Ldap Error: %s) on Server (%s)', $returnError, $ldapError, $ldapServer);
 				break;
 		}
 
@@ -344,6 +379,13 @@ class Ldap {
 		$this->version    = read_config_option('ldap_version');
 		$this->encryption = read_config_option('ldap_encryption');
 		$this->referrals  = read_config_option('ldap_referrals');
+		$this->debug      = read_config_option('ldap_debug');
+
+		if ($this->debug == 'on') {
+			$this->debug = POLLER_VERBOSITY_LOW;
+		} else {
+			$this->debug = POLLER_VERBOSITY_HIGH;
+		}
 
 		if (read_config_option('ldap_group_require') == 'on') {
 			$this->group_require = true;
@@ -369,7 +411,7 @@ class Ldap {
 		return true;
 	}
 
-	function ErrorHandler($level, $message, $file, $line, $context) {
+	function ErrorHandler($level, $message, $file, $line, $context = []) {
 		return true;
 	}
 
@@ -396,7 +438,7 @@ class Ldap {
 	function RecordError($output, $section = 'LDAP') {
 		$logDN = empty($output['dn']) ? '' : (', DN: ' . $output['dn']);
 		cacti_log($section . ': ' . $output['error_text'] . $logDN, false, 'AUTH');
-		cacti_log($section . ': ' . $output['stack'], false, 'AUTH', POLLER_VERBOSITY_HIGH);
+		cacti_log($section . ': ' . $output['stack'], false, 'AUTH', $this->debug);
 	}
 
 	function Connect() {
@@ -425,27 +467,27 @@ class Ldap {
 
 		/* Set debug if selective debug is enabled.  This places log data into the apache error_log */
 		if (get_selective_log_level() == POLLER_VERBOSITY_DEBUG) {
-			cacti_log('LDAP: Setting php-ldap into DEBUG mode.  Check your Web Server error_log for details', false, 'AUTH', POLLER_VERBOSITY_HIGH);
+			cacti_log('LDAP: Setting php-ldap into DEBUG mode.  Check your Web Server error_log for details', false, 'AUTH', $this->debug);
 			ldap_set_option(null, LDAP_OPT_DEBUG_LEVEL, 7);
 		}
 
 		if (getenv('TLS_CERT') != '' && defined('LDAP_OPT_X_TLS_CERTFILE')) {
-			cacti_log('NOTE: Settings TLS_CERT to ' . getenv('TLS_CERT'), false, 'AUTH', POLLER_VERBOSITY_HIGH);
+			cacti_log('NOTE: Settings TLS_CERT to ' . getenv('TLS_CERT'), false, 'AUTH', $this->debug);
 			ldap_set_option(null, LDAP_OPT_X_TLS_CERTFILE, getenv('TLS_CERT'));
 		}
 
 		if (getenv('TLS_CACERT') != '' && defined('LDAP_OPT_X_TLS_CACERTFILE')) {
-			cacti_log('NOTE: Settings TLS_CACERT to ' . getenv('TLS_CACERT'), false, 'AUTH', POLLER_VERBOSITY_HIGH);
+			cacti_log('NOTE: Settings TLS_CACERT to ' . getenv('TLS_CACERT'), false, 'AUTH', $this->debug);
 			ldap_set_option(null, LDAP_OPT_X_TLS_CACERTFILE, getenv('TLS_CACERT'));
 		}
 
 		if (getenv('TLS_KEY') != '' && defined('LDAP_OPT_X_TLS_KEYFILE')) {
-			cacti_log('NOTE: Settings TLS_KEY to ' . getenv('TLS_KEY'), false, 'AUTH', POLLER_VERBOSITY_HIGH);
+			cacti_log('NOTE: Settings TLS_KEY to ' . getenv('TLS_KEY'), false, 'AUTH', $this->debug);
 			ldap_set_option(null, LDAP_OPT_X_TLS_KEYFILE, getenv('TLS_KEY'));
 		}
 
 		if (getenv('TLS_CACERTDIR') != '' && defined('LDAP_OPT_X_TLS_CACERTDIR')) {
-			cacti_log('NOTE: Settings TLS_CACERTDIR to ' . getenv('TLS_CACERTDIR'), false, 'AUTH', POLLER_VERBOSITY_HIGH);
+			cacti_log('NOTE: Settings TLS_CACERTDIR to ' . getenv('TLS_CACERTDIR'), false, 'AUTH', $this->debug);
 			ldap_set_option(null, LDAP_OPT_X_TLS_CACERTDIR, getenv('TLS_CACERTDIR'));
 		}
 
@@ -458,23 +500,23 @@ class Ldap {
 			// For good measure, we will use both the php function and set the environment
 			switch($cert) {
 				case LDAP_OPT_X_TLS_NEVER:
-					cacti_log('NOTE: Setting TLS Certificate Requirement to \'never\'', false, 'AUTH', POLLER_VERBOSITY_HIGH);
+					cacti_log('NOTE: Setting TLS Certificate Requirement to \'never\'', false, 'AUTH', $this->debug);
 					putenv('TLS_REQCERT=never');
 					break;
 				case LDAP_OPT_X_TLS_HARD:
-					cacti_log('NOTE: Setting TLS Certificate Requirement to \'hard\'', false, 'AUTH', POLLER_VERBOSITY_HIGH);
+					cacti_log('NOTE: Setting TLS Certificate Requirement to \'hard\'', false, 'AUTH', $this->debug);
 					putenv('TLS_REQCERT=hard');
 					break;
 				case LDAP_OPT_X_TLS_DEMAND:
-					cacti_log('NOTE: Setting TLS Certificate Requirement to \'demand\'', false, 'AUTH', POLLER_VERBOSITY_HIGH);
+					cacti_log('NOTE: Setting TLS Certificate Requirement to \'demand\'', false, 'AUTH', $this->debug);
 					putenv('TLS_REQCERT=demand');
 					break;
 				case LDAP_OPT_X_TLS_ALLOW:
-					cacti_log('NOTE: Setting TLS Certificate Requirement to \'allow\'', false, 'AUTH', POLLER_VERBOSITY_HIGH);
+					cacti_log('NOTE: Setting TLS Certificate Requirement to \'allow\'', false, 'AUTH', $this->debug);
 					putenv('TLS_REQCERT=allow');
 					break;
 				case LDAP_OPT_X_TLS_TRY:
-					cacti_log('NOTE: Setting TLS Certificate Requirement to \'try\'', false, 'AUTH', POLLER_VERBOSITY_HIGH);
+					cacti_log('NOTE: Setting TLS Certificate Requirement to \'try\'', false, 'AUTH', $this->debug);
 					putenv('TLS_REQCERT=try');
 					break;
 			}
@@ -482,11 +524,12 @@ class Ldap {
 			ldap_set_option(null, LDAP_OPT_X_TLS_REQUIRE_CERT, $cert);
 		}
 
+		// Walk through ldap servers for a valid connections
 		if ($this->encryption == '1') {
-			cacti_log('LDAP: Connect using ldaps://' . $this->host . ':' . $this->port_ssl, false, 'AUTH', POLLER_VERBOSITY_HIGH);
+			cacti_log('LDAP: Connect using ldaps://' . $this->host . ':' . $this->port_ssl, false, 'AUTH', $this->debug);
 			$ldap_conn = ldap_connect('ldaps://' . $this->host . ':' . $this->port_ssl);
 		} else {
-			cacti_log('LDAP: Connect using ldap://'. $this->host . ':' . $this->port, false, 'AUTH', POLLER_VERBOSITY_HIGH);
+			cacti_log('LDAP: Connect using ldap://'. $this->host . ':' . $this->port, false, 'AUTH', $this->debug);
 			$ldap_conn = ldap_connect($this->host, $this->port);
 		}
 
@@ -495,7 +538,7 @@ class Ldap {
 			cacti_log('LDAP: Setting protocol version to ' . $this->version, false, 'AUTH', POLLER_VERBOSITY_MEDIUM);
 
 			if (!ldap_set_option($ldap_conn, LDAP_OPT_PROTOCOL_VERSION, $this->version)) {
-				$output = LdapError::GetErrorDetails(LdapError::ProtocolErrorVersion, $ldap_conn);
+				$output = LdapError::GetErrorDetails(LdapError::ProtocolErrorVersion, $ldap_conn, $this->host);
 				Ldap::RecordError($output);
 				ldap_close($ldap_conn);
 
@@ -505,10 +548,23 @@ class Ldap {
 				);
 			}
 
+			/* set reasonable timeouts */
+			$network_timeout = read_config_option('ldap_network_timeout');
+			if (defined('LDAP_OPT_NETWORK_TIMEOUT')) {
+				cacti_log("NOTE: Setting Network Timeout to $network_timeout seconds", false, 'AUTH', $this->debug);
+				ldap_set_option($ldap_conn, LDAP_OPT_NETWORK_TIMEOUT, $network_timeout);
+			}
+
+			$bind_timeout = read_config_option('ldap_bind_timeout');
+			if (defined('LDAP_OPT_TIMELIMIT')) {
+				cacti_log("NOTE: Setting Bind Timeout to $bind_timeout seconds", false, 'AUTH', $this->debug);
+				ldap_set_option($ldap_conn, LDAP_OPT_TIMEOUT, $bind_timeout);
+			}
+
 			/* set referrals */
 			if ($this->referrals == '0') {
 				if (!ldap_set_option($ldap_conn, LDAP_OPT_REFERRALS, 0)) {
-					$output = LdapError::GetErrorDetails(LdapError::ProtocolErrorReferral, $ldap_conn);
+					$output = LdapError::GetErrorDetails(LdapError::ProtocolErrorReferral, $ldap_conn, $this->host);
 					Ldap::RecordError($output);
 					ldap_close($ldap_conn);
 
@@ -522,7 +578,7 @@ class Ldap {
 			/* start TLS if requested */
 			if ($this->encryption == '2') {
 				if (!ldap_start_tls($ldap_conn)) {
-					$output = LdapError::GetErrorDetails(LdapError::ProtocolErrorTls, $ldap_conn);
+					$output = LdapError::GetErrorDetails(LdapError::ProtocolErrorTls, $ldap_conn, $this->host);
 					Ldap::RecordError($output);
 					ldap_close($ldap_conn);
 
@@ -550,7 +606,7 @@ class Ldap {
 			return $connection['output'];
 		} elseif ($connection['ldap_conn'] === false) {
 			$this->RestoreCactiHandler();
-			return LdapError::GetErrorDetails(LdapError::MissingLdapObject);
+			return LdapError::GetErrorDetails(LdapError::MissingLdapObject, false, $this->host);
 		}
 
 		$ldap_conn = $connection['ldap_conn'];
@@ -561,8 +617,12 @@ class Ldap {
 		$this->password = html_entity_decode($this->password, $this->GetMask(), 'UTF-8');
 		$this->dn = str_replace('<username>', $this->username, $this->dn);
 
+		if ($this->password == '') {
+			return LdapError::GetErrorDetails(LdapError::EmptyPassword);
+		}
+
 		/* Bind to the LDAP directory */
-		cacti_log('LDAP: Binding with "' . $this->dn . '"', false, 'AUTH', POLLER_VERBOSITY_HIGH);
+		cacti_log('LDAP: Binding with "' . $this->dn . '"', false, 'AUTH', $this->debug);
 		$ldap_response = ldap_bind($ldap_conn, $this->dn, $this->password);
 		if ($ldap_response) {
 			if ($this->group_require == 1) {
@@ -597,13 +657,13 @@ class Ldap {
 					/* Auth ok */
 					$output = LdapError::GetErrorDetails(LdapError::Success);
 				} elseif ($ldap_group_response === false) {
-					$output = LdapError::GetErrorDetails(LdapError::InsufficientAccess);
+					$output = LdapError::GetErrorDetails(LdapError::InsufficientAccess, $ldap_conn, $this->host);
 					Ldap::RecordError($output);
 					ldap_close($ldap_conn);
 					$this->RestoreCactiHandler();
 					return $output;
 				} else {
-					$output = LdapError::GetErrorDetails(LdapError::SearchFoundNoGroup);
+					$output = LdapError::GetErrorDetails(LdapError::SearchFoundNoGroup, $ldap_conn, $this->host);
 					Ldap::RecordError($output);
 					ldap_close($ldap_conn);
 					$this->RestoreCactiHandler();
@@ -618,22 +678,22 @@ class Ldap {
 			$ldap_error = ldap_errno($ldap_conn);
 			if ($ldap_error == 0x02) {
 				/* protocol error */
-				$output = LdapError::GetErrorDetails(LdapError::ProtocolErrorGeneral, $ldap_conn);
+				$output = LdapError::GetErrorDetails(LdapError::ProtocolErrorGeneral, $ldap_conn, $this->host);
 			} elseif ($ldap_error == 0x31) {
 				/* invalid credentials */
-				$output = LdapError::GetErrorDetails(LdapError::Failure);
+				$output = LdapError::GetErrorDetails(LdapError::Failure, $ldap_conn, $this->host);
 			} elseif ($ldap_error == 0x32) {
 				/* insuffient access */
-				$output = LdapError::GetErrorDetails(LdapError::InsufficientAccess);
+				$output = LdapError::GetErrorDetails(LdapError::InsufficientAccess, $ldap_conn, $this->host);
 			} elseif ($ldap_error == 0x51) {
 				/* unable to connect to server */
-				$output = LdapError::GetErrorDetails(LdapError::ConnectionUnavailable);
+				$output = LdapError::GetErrorDetails(LdapError::ConnectionUnavailable, $ldap_conn, $this->host);
 			} elseif ($ldap_error == 0x55) {
 				/* timeout */
-				$output = LdapError::GetErrorDetails(LdapError::ConnectionTimeout);
+				$output = LdapError::GetErrorDetails(LdapError::ConnectionTimeout, $ldap_conn, $this->host);
 			} else {
 				/* general bind error */
-				$output = LdapError::GetErrorDetails(LdapError::ProtocolErrorBind, $ldap_conn);
+				$output = LdapError::GetErrorDetails(LdapError::ProtocolErrorBind, $ldap_conn, $this->host);
 			}
 		}
 
@@ -667,10 +727,10 @@ class Ldap {
 
 		if (cacti_sizeof($connection['output'])) {
 			$this->RestoreCactiHandler();
-			return $output;
+			return $connection['output'];
 		} elseif ($connection['ldap_conn'] === false) {
 			$this->RestoreCactiHandler();
-			return LdapError::GetErrorDetails(LdapError::MissingLdapObject);
+			return LdapError::GetErrorDetails(LdapError::MissingLdapObject, false, $this->host);
 		}
 
 		$ldap_conn = $connection['ldap_conn'];
@@ -735,22 +795,22 @@ class Ldap {
 			$ldap_error = ldap_errno($ldap_conn);
 			if ($ldap_error == 0x02) {
 				/* protocol error */
-				$output = LdapError::GetErrorDetails(LdapError::ProtocolErrorGeneral, $ldap_conn);
+				$output = LdapError::GetErrorDetails(LdapError::ProtocolErrorGeneral, $ldap_conn, $this->host);
 			} elseif ($ldap_error == 0x31) {
 				/* invalid credentials */
-				$output = LdapError::GetErrorDetails(LdapError::Failure);
+				$output = LdapError::GetErrorDetails(LdapError::Failure, $ldap_conn, $this->host);
 			} elseif ($ldap_error == 0x32) {
 				/* insuffient access */
-				$output = LdapError::GetErrorDetails(LdapError::InsufficientAccess);
+				$output = LdapError::GetErrorDetails(LdapError::InsufficientAccess, $ldap_conn, $this->host);
 			} elseif ($ldap_error == 0x51) {
 				/* unable to connect to server */
-				$output = LdapError::GetErrorDetails(LdapError::ConnectionUnavailable);
+				$output = LdapError::GetErrorDetails(LdapError::ConnectionUnavailable, $ldap_conn, $this->host);
 			} elseif ($ldap_error == 0x55) {
 				/* timeout */
-				$output = LdapError::GetErrorDetails(LdapError::ConnectionTimeout);
+				$output = LdapError::GetErrorDetails(LdapError::ConnectionTimeout, $ldap_conn, $this->host);
 			} else {
 				/* general bind error */
-				$output = LdapError::GetErrorDetails(LdapError::ProtocolErrorBind, $ldap_conn);
+				$output = LdapError::GetErrorDetails(LdapError::ProtocolErrorBind, $ldap_conn, $this->host);
 			}
 		}
 
@@ -775,10 +835,10 @@ class Ldap {
 
 		if (cacti_sizeof($connection['output'])) {
 			$this->RestoreCactiHandler();
-			return $output;
+			return $connection['output'];
 		} elseif ($connection['ldap_conn'] === false) {
 			$this->RestoreCactiHandler();
-			return LdapError::GetErrorDetails(LdapError::MissingLdapObject);
+			return LdapError::GetErrorDetails(LdapError::MissingLdapObject, false, $this->host);
 		}
 
 		$ldap_conn = $connection['ldap_conn'];
@@ -847,22 +907,22 @@ class Ldap {
 			$ldap_error = ldap_errno($ldap_conn);
 			if ($ldap_error == 0x02) {
 				/* protocol error */
-				$output = LdapError::GetErrorDetails(LdapError::ProtocolErrorGeneral, $ldap_conn);
+				$output = LdapError::GetErrorDetails(LdapError::ProtocolErrorGeneral, $ldap_conn, $this->host);
 			} elseif ($ldap_error == 0x31) {
 				/* invalid credentials */
-				$output = LdapError::GetErrorDetails(LdapError::Failure);
+				$output = LdapError::GetErrorDetails(LdapError::Failure, $ldap_conn, $this->host);
 			} elseif ($ldap_error == 0x32) {
 				/* insuffient access */
-				$output = LdapError::GetErrorDetails(LdapError::InsufficientAccess);
+				$output = LdapError::GetErrorDetails(LdapError::InsufficientAccess, $ldap_conn, $this->host);
 			} elseif ($ldap_error == 0x51) {
 				/* unable to connect to server */
-				$output = LdapError::GetErrorDetails(LdapError::ConnectionUnavailable);
+				$output = LdapError::GetErrorDetails(LdapError::ConnectionUnavailable, $ldap_conn, $this->host);
 			} elseif ($ldap_error == 0x55) {
 				/* timeout */
-				$output = LdapError::GetErrorDetails(LdapError::ConnectionTimeout);
+				$output = LdapError::GetErrorDetails(LdapError::ConnectionTimeout, $ldap_conn, $this->host);
 			} else {
 				/* general bind error */
-				$output = LdapError::GetErrorDetails(LdapError::ProtocolErrorBind, $ldap_conn);
+				$output = LdapError::GetErrorDetails(LdapError::ProtocolErrorBind, $ldap_conn, $this->host);
 			}
 		}
 
