@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2021 The Cacti Group                                 |
+ | Copyright (C) 2004-2022 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -71,6 +71,94 @@ function duplicate_reports($_id, $_title) {
 	}
 }
 
+function reports_add_devices($report_id, $device_ids, $timespan, $align) {
+	$report_user = db_fetch_cell_prepared('SELECT user_id
+		FROM reports
+		WHERE id = ?',
+		array($report_id));
+
+	if ($report_user != $_SESSION['sess_user_id']) {
+		raise_message('reports_not_owner');
+
+		return false;
+	} else {
+		$errors = 0;
+
+		foreach($device_ids as $device_id) {
+			$sequence = db_fetch_cell_prepared('SELECT MAX(sequence)
+				FROM reports_items
+				WHERE report_id = ?',
+				array($report_id)) + 1;
+
+			$exists = db_fetch_cell_prepared('SELECT id
+				FROM reports_items
+				WHERE host_id = ?
+				AND item_type = 5
+				AND report_id = ?
+				AND timespan = ?',
+				array($device_id, $report_id, $timespan));
+
+			$device_exists = db_fetch_cell_prepared('SELECT id
+				FROM host
+				WHERE id = ?',
+				array($device_id));
+
+			$description = db_fetch_cell_prepared('SELECT description
+				FROM host
+				WHERE id = ?',
+				array($device_id));
+
+			if (!$exists) {
+				if ($device_exists > 0) {
+					$host_template_id = db_fetch_cell_prepared('SELECT host_template_id
+						FROM host
+						WHERE id = ?',
+						array($device_id));
+
+					$site_id = db_fetch_cell_prepared('SELECT site_id
+						FROM host
+						WHERE id = ?',
+						array($device_id));
+
+					// Setup defaults
+					$graph_template_id = -1;
+					$local_graph_id    = 0;
+
+					db_execute_prepared('INSERT INTO reports_items
+						(report_id, item_type, host_template_id, site_id, host_id, graph_template_id, local_graph_id, timespan, align, sequence)
+						VALUES (?, 5, ?, ?, ?, ?, ?, ?, ?, ?)',
+						array(
+							$report_id,
+							$host_template_id,
+							$site_id,
+							$device_id,
+							$graph_template_id,
+							$local_graph_id,
+							$timespan,
+							$align,
+							$sequence
+						)
+					);
+
+					raise_message('reports_add_device_' . $device_id, __('Device \'%s\' sucessfully added to Report.', $description), MESSAGE_LEVEL_INFO);
+				} else {
+					$errors++;
+					raise_message('reports_device_not_found', __('Device not found! Unable to add to Report'), MESSAGE_LEVEL_ERROR);
+				}
+			} else {
+				$errors++;
+				raise_message('reports_no_add_device_' . $device_id, __('Device \'%s\' not added to Report as it already exists on report.', $description), MESSAGE_LEVEL_WARN);
+			}
+		}
+
+		if ($errors) {
+			return false;
+		} else {
+			return true;
+		}
+	}
+}
+
 function reports_add_graphs($report_id, $local_graph_id, $timespan, $align) {
 	$report_user = db_fetch_cell_prepared('SELECT user_id
 		FROM reports
@@ -87,14 +175,14 @@ function reports_add_graphs($report_id, $local_graph_id, $timespan, $align) {
 			WHERE report_id = ?',
 			array($report_id)) + 1;
 
-		$existing = db_fetch_cell_prepared('SELECT id
+		$exists = db_fetch_cell_prepared('SELECT id
 			FROM reports_items
 			WHERE local_graph_id = ?
 			AND report_id = ?
 			AND timespan = ?',
 			array($local_graph_id, $report_id, $timespan));
 
-		if (!$existing) {
+		if (!$exists) {
 			$gd = db_fetch_row_prepared('SELECT *
 				FROM graph_local
 				WHERE id = ?',
@@ -311,7 +399,7 @@ function reports_log($string, $output = false, $environ = 'REPORTS', $level = PO
  * @param bool $force   - when forced, lastsent time will not be entered (e.g. Send Now)
  */
 function generate_report($report, $force = false) {
-	global $config, $alignment;
+	global $config, $alignment, $reports_interval, $attach_types;
 
 	include_once($config['base_path'] . '/lib/time.php');
 	include_once($config['base_path'] . '/lib/rrd.php');
@@ -545,10 +633,17 @@ function reports_load_format_file($format_file, &$output, &$report_tag_included,
 
 	$contents = array();
 
-	if (file_exists($config['base_path'] . '/formats/' . $format_file)) {
-		$contents = file($config['base_path'] . '/formats/' . $format_file);
+	if ($format_file == '') {
+		$format_file = 'cacti_group.format';
 	}
-	$output   = '';
+
+	$format_file = $config['base_path'] . '/formats/' . $format_file;
+
+	if (file_exists($format_file) && is_readable($format_file)) {
+		$contents = file($format_file);
+	}
+
+	$output = '';
 	$report_tag_included = false;
 
 	if (cacti_sizeof($contents)) {
@@ -720,8 +815,9 @@ function reports_generate_html($reports_id, $output = REPORTS_OUTPUT_STDOUT, &$t
 		}
 
 		$outstr .= "\t\t<tr class='title_row'>" . PHP_EOL;
+
 		if ($format_ok) {
-			$outstr .= "\t\t\t<td class='title' style='text-align:" . $alignment[$report['alignment']] . ";'>" . PHP_EOL;
+			$outstr .= "\t\t\t<td class='title'>" . PHP_EOL;
 		} else {
 			$outstr .= "\t\t\t<td class='title' style='text-align:" . $alignment[$report['alignment']] . ";font-size:" . $report['font_size'] . "pt;'>" . PHP_EOL;
 		}
@@ -751,7 +847,11 @@ function reports_generate_html($reports_id, $output = REPORTS_OUTPUT_STDOUT, &$t
 
 					if ($column == 0) {
 						$outstr .= "\t\t<tr class='image_row'>" . PHP_EOL;
-						$outstr .= "\t\t\t<td style='text-align:" . $alignment[$item['align']] . ";'>" . PHP_EOL;
+						if ($format_ok) {
+							$outstr .= "\t\t\t<td>" . PHP_EOL;
+						} else {
+							$outstr .= "\t\t\t<td style='text-align:" . $alignment[$item['align']] . ";'>" . PHP_EOL;
+						}
 
 						if ($format_ok) {
 							$outstr .= "\t\t\t\t<table class='image_table'>" . PHP_EOL;
@@ -763,7 +863,7 @@ function reports_generate_html($reports_id, $output = REPORTS_OUTPUT_STDOUT, &$t
 					}
 
 					if ($format_ok) {
-						$outstr .= "\t\t\t\t\t\t<td class='image_column' style='text-align:" . $alignment[$item['align']] . ";'>" . PHP_EOL;
+						$outstr .= "\t\t\t\t\t\t<td class='image_column'>" . PHP_EOL;
 					} else {
 						$outstr .= "\t\t\t\t\t\t<td style='padding:5px;text-align:" . $alignment[$item['align']] . ";'>" . PHP_EOL;
 					}
@@ -784,17 +884,23 @@ function reports_generate_html($reports_id, $output = REPORTS_OUTPUT_STDOUT, &$t
 				}
 			} elseif ($item['item_type'] == REPORTS_ITEM_TEXT) {
 				$outstr .= "\t\t<tr class='text_row'>" . PHP_EOL;
+
 				if ($format_ok) {
-					$outstr .= "\t\t\t<td style='text-align:" . $alignment[$item['align']] . ";' class='text'>" . PHP_EOL;
+					$outstr .= "\t\t\t<td class='text'>" . PHP_EOL;
 				} else {
 					$outstr .= "\t\t\t<td style='text-align:" . $alignment[$item['align']] . ";font-size: " . $item['font_size'] . "pt;' class='text'>" . PHP_EOL;
 				}
+
 				$outstr .= "\t\t\t\t" . html_escape($item['item_text']) . PHP_EOL;
 				$outstr .= "\t\t\t</td>" . PHP_EOL;
 				$outstr .= "\t\t</tr>" . PHP_EOL;
 
 				/* start a new section */
 				$column = 0;
+			} elseif ($item['item_type'] == REPORTS_ITEM_HOST) {
+				if (is_tree_allowed($item['host_id'], $report['user_id'])) {
+					$outstr .= reports_expand_device($report, $item, $item['host_id'], $output, $format_ok, $theme);
+				}
 			} elseif ($item['item_type'] == REPORTS_ITEM_TREE) {
 				if (is_tree_allowed($item['tree_id'], $report['user_id'])) {
 					if ($item['tree_cascade'] == 'on') {
@@ -809,6 +915,7 @@ function reports_generate_html($reports_id, $output = REPORTS_OUTPUT_STDOUT, &$t
 		}
 
 		$outstr .= "\t</table>" . PHP_EOL;
+
 		if ($output == REPORTS_OUTPUT_EMAIL && $include_body) {
 			$outstr .= '</body>';
 		}
@@ -893,14 +1000,160 @@ function expand_branch(&$report, &$item, $branch_id, $output, $format_ok, $theme
 	return $out . PHP_EOL;
 }
 
+/**
+ * expand a device for including into report
+ *
+ * @param array $report		- parameters for this report mail report
+ * @param int $item			- current graph item
+ * @param int $device_id    - the id of the host to include
+ * @param int $output		- type of output
+ * @param bool $format_ok	- use css styling
+ *
+ * @return string			- html
+ */
+function reports_expand_device(&$report, $item, $device_id, $output, $format_ok, $theme = 'classic') {
+	global $config, $alignment;
+
+	include($config['include_path'] . '/global_arrays.php');
+	include_once($config['library_path'] . '/data_query.php');
+	include_once($config['library_path'] . '/html_tree.php');
+	include_once($config['library_path'] . '/html_utility.php');
+
+	$time = time();
+
+	# get config option for first-day-of-the-week
+	$first_weekdayid = read_user_setting('first_weekdayid');
+
+	$user = $report['user_id'];
+
+	$timespan = array();
+
+	# get start/end time-since-epoch for actual time (now()) and given current-session-timespan
+	get_timespan($timespan, $time, $item['timespan'], $first_weekdayid);
+
+	$outstr = '';
+
+	$graphs = array();
+
+	$sql_where       = '';
+	$title_delimeter = '';
+	$search_key      = '';
+
+	$description = db_fetch_cell_prepared('SELECT h.description
+		FROM host AS h
+		WHERE h.id = ?',
+		array($device_id));
+
+	if ($description != '') {
+		$title = $title_delimeter . __('Device:') . " $description";
+		$title_delimeter = ' > ';
+	}
+
+	if ($item['graph_template_id'] == -1) {
+		$graph_templates = array_rekey(
+			db_fetch_assoc_prepared('SELECT DISTINCT
+				gt.id, gt.name
+				FROM graph_local AS gl
+				INNER JOIN graph_templates AS gt
+				ON gt.id=gl.graph_template_id
+				INNER JOIN graph_templates_graph AS gtg
+				ON gtg.local_graph_id=gl.id
+				WHERE gl.host_id = ?
+				ORDER BY gt.name',
+				array($device_id)),
+			'id', 'name'
+		);
+
+		/* for graphs without a template */
+		array_push($graph_templates,
+			array(
+				'id' => '0',
+				'name' => __('(No Graph Template)')
+			)
+		);
+	} else {
+		$graph_templates = array_rekey(
+			db_fetch_assoc_prepared('SELECT DISTINCT
+				gt.id, gt.name
+				FROM graph_templates AS gt
+				WHERE id = ?
+				ORDER BY gt.name',
+				array($item['graph_template_id'])),
+			'id', 'name'
+		);
+	}
+
+	if (cacti_sizeof($graph_templates)) {
+		foreach($graph_templates AS $id => $name) {
+			if (!is_graph_template_allowed($id, $user)) {
+				unset($graph_templates[$id]);
+			}
+		}
+	}
+
+	$outgraphs = array();
+	if (cacti_sizeof($graph_templates)) {
+		foreach ($graph_templates as $id => $name) {
+			if ($item['graph_name_regexp'] != '') {
+				$sql_where .= " AND title_cache REGEXP '" . $item['graph_name_regexp'] . "'";
+			}
+
+			$graphs = db_fetch_assoc_prepared("SELECT
+				gtg.local_graph_id, gtg.title_cache
+				FROM graph_local AS gl
+				INNER JOIN graph_templates_graph AS gtg
+				ON gtg.local_graph_id=gl.id
+				WHERE gl.graph_template_id = ?
+				AND gl.host_id = ?
+				$sql_where
+				ORDER BY gtg.title_cache",
+				array($id, $device_id));
+
+			if (cacti_sizeof($graphs)) {
+				foreach($graphs as $key => $graph) {
+					if (!is_graph_allowed($graph['local_graph_id'], $user)) {
+						unset($graphs[$key]);
+					}
+				}
+			}
+			$outgraphs = array_merge($outgraphs, $graphs);
+		}
+
+		if (cacti_sizeof($outgraphs)) {
+			/* let's sort the graphs naturally */
+			usort($outgraphs, 'necturally_sort_graphs');
+
+			/* start graph display */
+			if ($title != '') {
+				$outstr .= "\t\t<tr class='text_row'>" . PHP_EOL;
+
+				if ($format_ok) {
+					$outstr .= "\t\t\t<td class='text'>" . PHP_EOL;
+				} else {
+					$outstr .= "\t\t\t<td class='text' style='text-align:" . $alignment[$item['align']] . ";font-size: " . $item['font_size'] . "pt;'>" . PHP_EOL;
+				}
+
+				$outstr .= "\t\t\t\t$title" . PHP_EOL;
+				$outstr .= "\t\t\t</td>" . PHP_EOL;
+				$outstr .= "\t\t</tr>" . PHP_EOL;
+			}
+
+			$outstr .= reports_graph_area($outgraphs, $report, $item, $timespan, $output, $format_ok, $theme);
+		}
+	}
+
+	return $outstr;
+}
 
 /**
  * expand a tree for including into report
+ *
  * @param array $report		- parameters for this report mail report
  * @param int $item			- current graph item
  * @param int $output		- type of output
  * @param bool $format_ok	- use css styling
  * @param bool $nested		- nested tree?
+ *
  * @return string			- html
  */
 function reports_expand_tree(&$report, $item, $parent, $output, $format_ok, $theme = 'classic', $nested = false) {
@@ -1012,23 +1265,23 @@ function reports_expand_tree(&$report, $item, $parent, $output, $format_ok, $the
 			}
 
 			if (!empty($tree_name) && empty($leaf_name) && empty($host_name)) {
-				$title = $title_delimeter . '<strong>' . __('Tree:') . "</strong> $tree_name";
-				$title_delimeter = '-> ';
+				$title = $title_delimeter . __('Tree:') . " $tree_name";
+				$title_delimeter = ' > ';
 			}
 
 			if (!empty($leaf_name)) {
-				$title .= $title_delimeter . '<strong>' . __('Leaf:') . "</strong> $leaf_name";
-				$title_delimeter = '-> ';
+				$title .= $title_delimeter . " $leaf_name";
+				$title_delimeter = ' > ';
 			}
 
 			if (!empty($host_name)) {
-				$title .= $title_delimeter . '<strong>' . __('Host:') . "</strong> $host_name";
-				$title_delimeter = '-> ';
+				$title .= $title_delimeter . " $host_name";
+				$title_delimeter = ' > ';
 			}
 
 			if (!empty($graph_name) && !$nested) {
-				$title .= $title_delimeter . '<strong>' . __('Graph:') . "</strong> $graph_name";
-				$title_delimeter = '-> ';
+				$title .= $title_delimeter . " $graph_name";
+				$title_delimeter = ' > ';
 			}
 
 			if ($item['graph_name_regexp'] != '') {
@@ -1062,7 +1315,7 @@ function reports_expand_tree(&$report, $item, $parent, $output, $format_ok, $the
 					if ($title != '') {
 						$outstr .= "\t\t<tr class='text_row'>" . PHP_EOL;
 						if ($format_ok) {
-							$outstr .= "\t\t\t<td class='text' style='text-align:" . $alignment[$item['align']] . "'>" . PHP_EOL;
+							$outstr .= "\t\t\t<td class='text'>" . PHP_EOL;
 						} else {
 							$outstr .= "\t\t\t<td class='text' style='text-align:" . $alignment[$item['align']] . ";font-size: " . $item['font_size'] . "pt;'>" . PHP_EOL;
 						}
@@ -1103,7 +1356,7 @@ function reports_expand_tree(&$report, $item, $parent, $output, $format_ok, $the
 					if ($title != '') {
 						$outstr .= "\t\t<tr class='text_row'>" . PHP_EOL;
 						if ($format_ok) {
-							$outstr .= "\t\t\t<td class='text' style='text-align:" . $alignment[$item['align']] . ";'>" . PHP_EOL;
+							$outstr .= "\t\t\t<td class='text'>" . PHP_EOL;
 						} else {
 							$outstr .= "\t\t\t<td class='text' style='text-align:" . $alignment[$item['align']] . ";font-size: " . $item['font_size'] . "pt;'>" . PHP_EOL;
 						}
@@ -1116,7 +1369,6 @@ function reports_expand_tree(&$report, $item, $parent, $output, $format_ok, $the
 				$graph_list = array(array('local_graph_id' => $leaf['local_graph_id'], 'title_cache' => $graph_name));
 
 				$outstr .= reports_graph_area($graph_list, $report, $item, $timespan, $output, $format_ok, $theme);
-			//} elseif ($leaf_type == 'host' && $nested) {
 			} elseif ($leaf_type == 'host') {
 				if ($item['tree_cascade'] == 'on') {
 					/* graph template grouping */
@@ -1183,7 +1435,7 @@ function reports_expand_tree(&$report, $item, $parent, $output, $format_ok, $the
 									$outstr .= "\t\t<tr class='text_row'>" . PHP_EOL;
 
 									if ($format_ok) {
-										$outstr .= "\t\t\t<td class='text' style='text-align:" . $alignment[$item['align']] . "';>" . PHP_EOL;
+										$outstr .= "\t\t\t<td class='text'>" . PHP_EOL;
 									} else {
 										$outstr .= "\t\t\t<td class='text' style='text-align:" . $alignment[$item['align']] . ";font-size: " . $item['font_size'] . "pt;'>" . PHP_EOL;
 									}
@@ -1249,7 +1501,7 @@ function reports_expand_tree(&$report, $item, $parent, $output, $format_ok, $the
 										if ($title != '') {
 											$outstr .= "\t\t<tr class='text_row'>" . PHP_EOL;
 											if ($format_ok) {
-												$outstr .= "\t\t\t<td class='text' style='text-align:" . $alignment[$item['align']] . ";'>" . PHP_EOL;
+												$outstr .= "\t\t\t<td class='text'>" . PHP_EOL;
 											} else {
 												$outstr .= "\t\t\t<td class='text' style='text-align:" . $alignment[$item['align']] . ";font-size: " . $item['font_size'] . "pt;'>" . PHP_EOL;
 											}
@@ -1262,11 +1514,11 @@ function reports_expand_tree(&$report, $item, $parent, $output, $format_ok, $the
 
 									$outstr .= "\t\t<tr class='text_row'>" . PHP_EOL;
 									if ($format_ok) {
-										$outstr .= "\t\t\t<td class='text' style='text-align:" . $alignment[$item['align']] . ";'><strong>" . __('Data Query:') . "</strong> " . $data_query['name'] . PHP_EOL;
+										$outstr .= "\t\t\t<td class='text'>" . __('Data Query:') . ' ' . $data_query['name'] . PHP_EOL;
 										$outstr .= "\t\t\t</td>" . PHP_EOL;
 										$outstr .= "\t\t</tr>" . PHP_EOL;
 									} else {
-										$outstr .= "\t\t\t<td class='text' style='text-align:" . $alignment[$item['align']] . ";font-size: " . $item['font_size'] . "pt;'><strong>" . __('Data Query:') . "</strong> " . $data_query['name'] . PHP_EOL;
+										$outstr .= "\t\t\t<td class='text' style='text-align:" . $alignment[$item['align']] . ";font-size: " . $item['font_size'] . "pt;'>" . __('Data Query:') . ' ' . $data_query['name'] . PHP_EOL;
 										$outstr .= "\t\t\t</td>" . PHP_EOL;
 										$outstr .= "\t\t</tr>" . PHP_EOL;
 									}
@@ -1320,7 +1572,7 @@ function reports_expand_tree(&$report, $item, $parent, $output, $format_ok, $the
 						if ($title != '') {
 							$outstr .= "\t\t<tr class='text_row'>" . PHP_EOL;
 							if ($format_ok) {
-								$outstr .= "\t\t\t<td class='text' style='text-align:" . $alignment[$item['align']] . ";'>" . PHP_EOL;
+								$outstr .= "\t\t\t<td class='text'>" . PHP_EOL;
 							} else {
 								$outstr .= "\t\t\t<td class='text' style='text-align:" . $alignment[$item['align']] . ";font-size: " . $item['font_size'] . "pt;'>" . PHP_EOL;
 							}
@@ -1340,7 +1592,7 @@ function reports_expand_tree(&$report, $item, $parent, $output, $format_ok, $the
 			if ($title != '') {
 				$outstr .= "\t\t<tr class='text_row'>" . PHP_EOL;
 				if ($format_ok) {
-					$outstr .= "\t\t\t<td class='text' style='text-align:" . $alignment[$item['align']] . "'>" . PHP_EOL;
+					$outstr .= "\t\t\t<td class='text'>" . PHP_EOL;
 				} else {
 					$outstr .= "\t\t\t<td class='text' style='text-align:" . $alignment[$item['align']] . ";font-size: " . $item['font_size'] . "pt;'>" . PHP_EOL;
 				}
@@ -1356,7 +1608,6 @@ function reports_expand_tree(&$report, $item, $parent, $output, $format_ok, $the
 	return $outstr;
 }
 
-
 /**
  * natural sort function
  * @param $a
@@ -1390,13 +1641,20 @@ function reports_graph_area($graphs, &$report, $item, $timespan, $output, $forma
 
 			if ($column == 0) {
 				$outstr .= "\t\t<tr class='image_row'>" . PHP_EOL;
-				$outstr .= "\t\t\t<td style='text-align:" . $alignment[$item['align']] . ";'>" . PHP_EOL;
-				$outstr .= "\t\t\t\t<table style='width:100%;'>" . PHP_EOL;
+
+				if ($format_ok) {
+					$outstr .= "\t\t\t<td>" . PHP_EOL;
+					$outstr .= "\t\t\t\t<table>". PHP_EOL;
+				} else {
+					$outstr .= "\t\t\t<td style='text-align:" . $alignment[$item['align']] . ";'>" . PHP_EOL;
+					$outstr .= "\t\t\t\t<table style='width:100%;'>" . PHP_EOL;
+				}
+
 				$outstr .= "\t\t\t\t\t<tr>" . PHP_EOL;
 			}
 
 			if ($format_ok) {
-				$outstr .= "\t\t\t\t\t\t<td class='image_column' style='text-align:" . $alignment[$item['align']] . ";'>" . PHP_EOL;
+				$outstr .= "\t\t\t\t\t\t<td class='image_column'>" . PHP_EOL;
 			} else {
 				$outstr .= "\t\t\t\t\t\t<td style='padding:5px;text-align='" . $alignment[$item['align']] . ";'>" . PHP_EOL;
 			}
@@ -1643,14 +1901,14 @@ function reports_graphs_action_execute($action) {
 
 				foreach($selected_items as $local_graph_id) {
 					/* see if the graph is already added */
-					$existing = db_fetch_cell_prepared('SELECT id
+					$exists = db_fetch_cell_prepared('SELECT id
 						FROM reports_items
 						WHERE local_graph_id = ?
 						AND report_id = ?
 						AND timespan = ?',
 						array($local_graph_id, $reports_id, get_nfilter_request_var('timespan')));
 
-					if (!$existing) {
+					if (!$exists) {
 						$sequence = db_fetch_cell_prepared('SELECT MAX(sequence)
 							FROM reports_items
 							WHERE report_id = ?',
@@ -1709,4 +1967,3 @@ function reports_graphs_action_execute($action) {
 		return $action;
 	}
 }
-
