@@ -2,7 +2,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2021 The Cacti Group                                 |
+ | Copyright (C) 2004-2022 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -156,8 +156,8 @@ if ($child == false) {
 			boost_launch_children();
 
 			/* Wait for all processes to continue */
-			while (boost_processes_running()) {
-				boost_debug('Sleeping for 2 seconds.');
+			while ($running = boost_processes_running()) {
+				boost_debug(sprintf('%s Processes Runnning, Sleeping for 2 seconds.', $running));
 				sleep(2);
 			}
 
@@ -278,16 +278,12 @@ function boost_kill_running_processes() {
 }
 
 function boost_processes_running() {
-	$finished = db_fetch_cell('SELECT COUNT(*)
+	$running = db_fetch_cell('SELECT COUNT(*)
 		FROM processes
 		WHERE tasktype = "boost"
 		AND taskname = "child"');
 
-	if ($finished == 0) {
-		return false;
-	}
-
-	return true;
+	return $running;
 }
 
 function boost_prepare_process_table() {
@@ -307,6 +303,8 @@ function boost_prepare_process_table() {
 			/* if the runtime was exceeded, allow the next process to run */
 			if ($previous_start_time + $max_run_duration < $start_time) {
 				cacti_log('WARNING: Detected Poller Boost Overrun, Possible Boost Poller Crash', false, 'BOOST SVR');
+				admin_email(__('Cacti System Warning'), __('WARNING: Detected Poller Boost Overrun, Possible Boost Poller Crash', 'BOOST SVR'  ));
+
 			}
 		}
 	}
@@ -331,6 +329,7 @@ function boost_prepare_process_table() {
 
 	db_execute("CREATE TABLE $interim_table LIKE poller_output_boost");
 	db_execute("RENAME TABLE poller_output_boost TO $archive_table, $interim_table TO poller_output_boost");
+	db_execute("ANALYZE TABLE $archive_table");
 
 	$arch_tables = boost_get_arch_table_names();
 
@@ -412,11 +411,12 @@ function boost_launch_children() {
 
 	$php_binary    = read_config_option('path_php_binary');
 	$boost_log     = read_config_option('path_boost_log');
+	$boost_logdir  = dirname($boost_log);
 	$redirect_args = '';
 
 	if ($boost_log != '') {
-		if (!is_writable($boost_log)) {
-			boost_debug("WARNING: Boost log '$boost_log' is not writable!");
+		if (!is_writable($boost_log) || !is_dir($boost_logdir) || !is_writable($boost_logdir)) {
+			boost_debug("WARNING: Boost log '$boost_log' does not exist or is not writable!");
 
 			cacti_log("WARNING: Boost log '$boost_log' is not writable!", false, 'BOOST');
 		} else {
@@ -458,8 +458,8 @@ function boost_time_to_run($forcerun, $current_time, $last_run_time, $next_run_t
 
 	if ((read_config_option('boost_rrd_update_enable') == 'on') || $forcerun) {
 		/* turn on the system level updates as that is what dictates "on/off" */
-		if ((!$forcerun) && (read_config_option('boost_rrd_update_system_enable') != 'on')) {
-			set_config_option('boost_rrd_update_system_enable','on');
+		if (!$forcerun && read_config_option('boost_rrd_update_system_enable') != 'on') {
+			set_config_option('boost_rrd_update_system_enable', 'on');
 		}
 
 		$seconds_offset = read_config_option('boost_rrd_update_interval') * 60;
