@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2021 The Cacti Group                                 |
+ | Copyright (C) 2004-2022 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -170,6 +170,15 @@ $fields_poller_edit = array(
 		'default' => $database_port,
 		'max_length' => '5'
 	),
+	'dbretries' => array(
+		'method' => 'textbox',
+		'friendly_name' => __('Remote Database Retries'),
+		'description' => __('The number of times to attempt to retry to connect to the remote database.'),
+		'value' => '|arg1:dbretries|',
+		'size' => '5',
+		'default' => $database_retries,
+		'max_length' => '5'
+	),
 	'dbssl' => array(
 		'method' => 'checkbox',
 		'friendly_name' => __('Remote Database SSL'),
@@ -285,7 +294,8 @@ function form_save() {
 			$save['dbhost']        = form_input_validate(get_nfilter_request_var('dbhost'),    'dbhost',    '', true, 3);
 			$save['dbuser']        = form_input_validate(get_nfilter_request_var('dbuser'),    'dbuser',    '', true, 3);
 			$save['dbpass']        = form_input_validate(get_nfilter_request_var('dbpass'),    'dbpass',    '', true, 3);
-			$save['dbport']        = form_input_validate(get_nfilter_request_var('dbport'),    'dbport',    '', true, 3);
+			$save['dbport']        = form_input_validate(get_nfilter_request_var('dbport'),    'dbport',    '^[0-9]+$', true, 3);
+			$save['dbretries']     = form_input_validate(get_nfilter_request_var('dbretries'), 'dbretries', '^[0-9]+$', true, 3);
 			$save['dbssl']         = isset_request_var('dbssl') ? 'on':'';
 			$save['dbsslkey']      = form_input_validate(get_nfilter_request_var('dbsslkey'),  'dbsslkey',  '', true, 3);
 			$save['dbsslcert']     = form_input_validate(get_nfilter_request_var('dbsslcert'), 'dbsslcert', '', true, 3);
@@ -382,15 +392,23 @@ function poller_check_duplicate_poller_id($poller_id, $hostname, $column) {
 	if (cacti_sizeof($ip_hostnames)) {
 		foreach($ip_hostnames as $host) {
 			$parts = explode('.', $host);
-			$sql_where2 .= ($sql_where2 != '' ? ' OR ' : ($sql_where1 != '' ? ' OR ' : '') . ' (') . "($column = '$parts[0]' OR $column LIKE '$parts[0].%' OR $column = '$host')";
+			$sql_where2 .= ($sql_where2 != '' ? ' OR ':' (') .
+				"($column = " . db_qstr($parts[0]) .
+				" OR $column = " . db_qstr($host) . ")";
 		}
 		$sql_where2 .= ')';
 	}
 
+	if ($sql_where1 != '' || $sql_where2 != '') {
+		$sql_where = ' AND (' . $sql_where1 . ($sql_where1 != '' && $sql_where2 != '' ? ' OR ':'') . $sql_where2 . ')';
+	} else {
+		$sql_where = '';
+	}
+
 	$duplicate = db_fetch_cell_prepared("SELECT id
 		FROM poller
 		WHERE id != ?
-		AND ($sql_where1 $sql_where2)",
+		$sql_where",
 		array($poller_id));
 
 	if (empty($duplicate)) {
@@ -635,6 +653,7 @@ function poller_edit() {
 			unset($fields_poller_edit['dbuser']);
 			unset($fields_poller_edit['dbpass']);
 			unset($fields_poller_edit['dbport']);
+			unset($fields_poller_edit['dbretries']);
 			unset($fields_poller_edit['dbssl']);
 			unset($fields_poller_edit['dbsslkey']);
 			unset($fields_poller_edit['dbsslcert']);
@@ -712,6 +731,7 @@ function poller_edit() {
 					dbuser:       $('#dbuser').val(),
 					dbpass:       $('#dbpass').val(),
 					dbport:       $('#dbport').val(),
+					dbretries:    $('#dbretries').val(),
 					dbssl:        dbssl,
 					dbsslkey:     $('#dbsslkey').val(),
 					dbsslcert:    $('#dbsslcert').val(),
@@ -746,7 +766,19 @@ function test_database_connection($poller = array()) {
 	if (!cacti_sizeof($poller)) {
 		$poller['dbtype'] = 'mysql';
 
-		$fields = array('dbhost', 'dbuser', 'dbpass', 'dbdefault', 'dbport', 'dbssl', 'dbsslkey', 'dbsslcert', 'dbsslca');
+		$fields = array(
+			'dbhost',
+			'dbuser',
+			'dbpass',
+			'dbdefault',
+			'dbport',
+			'dbretries',
+			'dbssl',
+			'dbsslkey',
+			'dbsslcert',
+			'dbsslca'
+		);
+
 		foreach ($fields as $field) {
 			if ($field == 'dbssl') {
 				if (isset_request_var('dbssl') && get_nfilter_request_var('dbssl') == 'on') {
@@ -770,6 +802,7 @@ function test_database_connection($poller = array()) {
 		$poller['dbdefault'],
 		$poller['dbtype'],
 		$poller['dbport'],
+		$poller['dbretries'],
 		$poller['dbssl'],
 		$poller['dbsslkey'],
 		$poller['dbsslcert'],
@@ -990,7 +1023,7 @@ function pollers() {
 				$poller['status'] = 6;
 			}
 
-			$mma = round($poller['avg_time'], 2) . '/' .  round($poller['max_time'], 2);
+			$mma = round($poller['avg_time']?:0, 2) . '/' .  round(max($poller['max_time']?:1,1), 2);
 
 			if (empty($poller['name'])) {
 				$poller['name'] = '&lt;no name&gt;';
