@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2020 The Cacti Group                                 |
+ | Copyright (C) 2004-2022 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -22,9 +22,15 @@
  +-------------------------------------------------------------------------+
 */
 
-require(__DIR__ . '/include/cli_check.php');
+if (function_exists('pcntl_async_signals')) {
+	pcntl_async_signals(true);
+} else {
+	declare(ticks = 100);
+}
 
-declare(ticks = 1);
+ini_set('output_buffering', 'Off');
+
+require(__DIR__ . '/include/cli_check.php');
 
 /* need to capture signals from users */
 function sig_handler($signo) {
@@ -63,7 +69,7 @@ if (function_exists('pcntl_signal')) {
 	pcntl_signal(SIGSEGV, 'sig_handler');
 }
 
-error_reporting(0);
+error_reporting(E_ALL);
 
 /* define STDOUT/STDIN file descriptors if not running under CLI */
 if (php_sapi_name() != 'cli') {
@@ -71,6 +77,9 @@ if (php_sapi_name() != 'cli') {
 	define('STDOUT', fopen('php://stdout', 'w'));
 }
 
+/* make sure data is flushed immediately */
+ob_implicit_flush();
+
 /* signal for realtime */
 global $environ, $poller_id;
 
@@ -80,15 +89,14 @@ $start = microtime(true);
 /* some debugging */
 $pid = getmypid();
 $ctr = 0;
+$called_by_script_server = false;
 
 /* if multiple polling intervals are defined, compensate for them */
 $polling_interval = read_config_option('poller_interval');
 
 if (!empty($polling_interval)) {
-	$num_polling_items = db_fetch_cell_prepared('SELECT count(*) FROM poller_item WHERE rrd_next_step <= 0 AND poller_id = ?', array($config['poller_id']), 'count(*)');
 	define('MAX_POLLER_RUNTIME', $polling_interval);
 } else {
-	$num_polling_items = db_fetch_cell_prepared('SELECT count(*) FROM poller_item WHERE poller_id = ?', array($config['poller_id']), 'count(*)');
 	define('MAX_POLLER_RUNTIME', 300);
 }
 
@@ -97,8 +105,13 @@ ini_set('max_execution_time', MAX_POLLER_RUNTIME + 1);
 
 /* Record the calling environment */
 if ($_SERVER['argc'] >= 2) {
-	if ($_SERVER['argv'][1] == 'spine')
+	if (substr( $_SERVER['argv'][1], 0, 5) == 'spine')
+		global $force_level;
 		$environ = 'spine';
+		if (strlen($_SERVER['argv'][1]) > 5) {
+			$force_level = intval(substr($_SERVER['argv'][1], 5));
+			cacti_log('INFO: Forcing log level to: ' . $force_level, false, 'PHPSVR', POLLER_VERBOSITY_HIGH);
+		}
 	else
 		if (($_SERVER['argv'][1] == 'cmd.php') || ($_SERVER['argv'][1] == 'cmd'))
 			$environ = 'cmd';
@@ -150,7 +163,7 @@ while (1) {
 				$out = [];
 				exec("TASKLIST /FO LIST /FI \"PID eq $parent_pid\"", $out);
 
-				$isParentRunning = (count($out) > 1);
+				$isParentRunning = (cacti_count($out) > 1);
 			} elseif (function_exists('posix_kill')) {
 				$isParentRunning = posix_kill(intval($parent_pid), 0);
 			}
@@ -172,9 +185,11 @@ while (1) {
 		$input_string = trim($input_string);
 
 		if (substr($input_string,0,4) == 'quit') {
-			fputs(STDOUT, 'PHP Script Server Shutdown request received, exiting' . PHP_EOL);
-			fflush(STDOUT);
-			cacti_log('DEBUG: PHP Script Server Shutdown request received, exiting', false, 'PHPSVR', POLLER_VERBOSITY_DEBUG);
+			if (!$called_by_script_server) {
+				fputs(STDOUT, 'PHP Script Server Shutdown request received, exiting' . PHP_EOL);
+				fflush(STDOUT);
+				cacti_log('DEBUG: PHP Script Server Shutdown request received, exiting', false, 'PHPSVR', POLLER_VERBOSITY_DEBUG);
+			}
 			db_close();
 			exit(0);
 		}
@@ -217,8 +232,8 @@ while (1) {
 			}
 
 			cacti_log("DEBUG: PID[$pid] CTR[$ctr] INC: '". basename($include_file) .
-			        "' FUNC: '$function' PARMS: '" . implode('\', \'',$parameter_array) .
-			        "'", false, 'PHPSVR', POLLER_VERBOSITY_DEBUG);
+				"' FUNC: '$function' PARMS: '" . implode('\', \'',$parameter_array) .
+				"'", false, 'PHPSVR', POLLER_VERBOSITY_DEBUG);
 
 			/* validate the existance of the function, and include if applicable */
 			if (!function_exists($function)) {
@@ -321,7 +336,9 @@ function parseArgs($string, &$str_list, $debug = false) {
 
 			break;
 		case '\\':
-			if ($escaping) {
+			if ($indelim) {
+				$curstr  .= $char;
+			} elseif ($escaping) {
 				$curstr  .= $char;
 				$escaping = false;
 			} else {
