@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2021 The Cacti Group                                 |
+ | Copyright (C) 2004-2022 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -203,6 +203,12 @@ case 'save':
 					array($field_name, get_nfilter_request_var($field_name)));
 			}
 		}
+
+		if ($field_name == 'auth_method') {
+			if (get_nfilter_request_var($field_name) == '2') {
+				db_execute('TRUNCATE TABLE user_auth_cache');
+			}
+		}
 	}
 
 	if (isset_request_var('log_verbosity')) {
@@ -237,13 +243,13 @@ case 'save':
 	$gone_time = read_config_option('poller_interval') * 2;
 
 	$pollers = array_rekey(
-		db_fetch_assoc_prepared('SELECT id
+		db_fetch_assoc('SELECT
+			id,
+			UNIX_TIMESTAMP() - UNIX_TIMESTAMP(last_status) AS last_polled
 			FROM poller
 			WHERE id > 1
-			AND disabled=""
-			AND UNIX_TIMESTAMP() - UNIX_TIMESTAMP(last_status) <= ?',
-			array($gone_time)),
-		'id', 'id'
+			AND disabled=""'),
+		'id', 'last_polled'
 	);
 
 	if (get_request_var('tab') == 'path' && $config['poller_id'] > 1) {
@@ -257,11 +263,20 @@ case 'save':
 				VALUES ' . implode(', ', $inserts) . '
 				ON DUPLICATE KEY UPDATE value=VALUES(value)';
 
-			foreach($pollers as $p) {
-				$rcnn_id = poller_connect_to_remote($p);
+			foreach($pollers as $p => $t) {
+				if ($t > $gone_time) {
+					raise_message('poller_' . $p, __('Settings save to Data Collector %d skipped due to heartbeat.', $p), MESSAGE_LEVEL_WARN);
+				} else {
+					$rcnn_id = poller_connect_to_remote($p);
 
-				if ($rcnn_id) {
-					if (db_execute($sql, false, $rcnn_id) === false) {
+					if ($rcnn_id) {
+						if (db_execute($sql, false, $rcnn_id) === false) {
+							$rcnn_id = false;
+						}
+					}
+
+					// check if we still have rcnn_id, if it's now become false, we had a problem
+					if (!$rcnn_id) {
 						raise_message('poller_' . $p, __('Settings save to Data Collector %d Failed.', $p), MESSAGE_LEVEL_ERROR);
 					}
 				}
@@ -500,7 +515,9 @@ default:
 
 		if (currentTab == 'general') {
 			$('#selective_plugin_debug').multiselect({
-				height: 300,
+				menuHeight: $(window).height()*.7,
+				menuWidth: 'auto',
+				linkInfo: faIcons,
 				noneSelectedText: '<?php print __('Select Plugin(s)');?>',
 				selectedText: function(numChecked, numTotal, checkedItems) {
 					myReturn = numChecked + ' <?php print __('Plugins Selected');?>';
@@ -520,6 +537,9 @@ default:
 			});
 
 			$('#selective_debug').multiselect({
+				menuHeight: $(window).height()*.7,
+				menuWidth: 'auto',
+				linkInfo: faIcons,
 				noneSelectedText: '<?php print __('Select File(s)');?>',
 				selectedText: function(numChecked, numTotal, checkedItems) {
 					myReturn = numChecked + ' <?php print __('Files Selected');?>';
@@ -527,7 +547,7 @@ default:
 				},
 				checkAllText: '<?php print __('All');?>',
 				uncheckAllText: '<?php print __('None');?>',
-				uncheckall: function() {
+				uncheckAll: function() {
 					$(this).multiselect('widget').find(':checkbox:first').each(function() {
 						$(this).prop('checked', true);
 					});
@@ -539,7 +559,9 @@ default:
 			});
 		} else if (currentTab == 'spikes') {
 			$('#spikekill_templates').multiselect({
-				height: 300,
+				menuHeight: $(window).height()*.7,
+				menuWidth: 'auto',
+				linkInfo: faIcons,
 				noneSelectedText: '<?php print __('Select Template(s)');?>',
 				selectedText: function(numChecked, numTotal, checkedItems) {
 					myReturn = numChecked + ' <?php print __('Templates Selected');?>';
@@ -553,7 +575,7 @@ default:
 				},
 				checkAllText: '<?php print __('All');?>',
 				uncheckAllText: '<?php print __('None');?>',
-				uncheckall: function() {
+				uncheckAll: function() {
 					$(this).multiselect('widget').find(':checkbox:first').each(function() {
 						$(this).prop('checked', true);
 					});
@@ -982,6 +1004,12 @@ default:
 			$('#row_secpass_lock_header').hide();
 			$('#row_secpass_lockfailed').hide();
 			$('#row_secpass_unlocktime').hide();
+			$('#row_basic_header').hide();
+			$('#row_basic_auth_fail_message').hide();
+			$('#row_path_basic_mapfile').hide();
+			$('#row_ldap_network_timeout').hide();
+			$('#row_ldap_bind_timeout').hide();
+			$('#row_ldap_tls_certificate').hide();
 			break;
 		case '1': // Builtin
 			$('#row_path_basic_mapfile').hide();
@@ -1025,6 +1053,13 @@ default:
 			$('#row_secpass_lock_header').show();
 			$('#row_secpass_lockfailed').show();
 			$('#row_secpass_unlocktime').show();
+			$('#row_basic_header').hide();
+			$('#row_basic_auth_fail_message').hide();
+			$('#row_path_basic_mapfile').hide();
+			$('#row_ldap_network_timeout').hide();
+			$('#row_ldap_bind_timeout').hide();
+			$('#row_ldap_tls_certificate').hide();
+			$('#row_ldap_debug').hide();
 			break;
 		case '2': // Web Basic
 			$('#row_path_basic_mapfile').show();
@@ -1068,6 +1103,13 @@ default:
 			$('#row_secpass_lock_header').hide();
 			$('#row_secpass_lockfailed').hide();
 			$('#row_secpass_unlocktime').hide();
+			$('#row_basic_header').show();
+			$('#row_basic_auth_fail_message').show();
+			$('#row_path_basic_mapfile').show();
+			$('#row_ldap_network_timeout').hide();
+			$('#row_ldap_bind_timeout').hide();
+			$('#row_ldap_tls_certificate').hide();
+			$('#row_ldap_debug').hide();
 			break;
 		case '4': // Multiple Domains
 			$('#row_path_basic_mapfile').hide();
@@ -1111,6 +1153,13 @@ default:
 			$('#row_secpass_lock_header').show();
 			$('#row_secpass_lockfailed').show();
 			$('#row_secpass_unlocktime').show();
+			$('#row_basic_header').hide();
+			$('#row_basic_auth_fail_message').hide();
+			$('#row_path_basic_mapfile').hide();
+			$('#row_ldap_network_timeout').show();
+			$('#row_ldap_bind_timeout').show();
+			$('#row_ldap_tls_certificate').show();
+			$('#row_ldap_debug').show();
 			break;
 		case '3': // Single Domain
 			$('#row_special_users_header').show();
@@ -1153,6 +1202,13 @@ default:
 			$('#row_secpass_lock_header').show();
 			$('#row_secpass_lockfailed').show();
 			$('#row_secpass_unlocktime').show();
+			$('#row_basic_header').hide();
+			$('#row_basic_auth_fail_message').hide();
+			$('#row_path_basic_mapfile').hide();
+			$('#row_ldap_network_timeout').show();
+			$('#row_ldap_bind_timeout').show();
+			$('#row_ldap_tls_certificate').show();
+			$('#row_ldap_debug').show();
 			initSearch();
 			initGroupMember();
 			break;
@@ -1197,6 +1253,13 @@ default:
 			$('#row_secpass_lock_header').show();
 			$('#row_secpass_lockfailed').show();
 			$('#row_secpass_unlocktime').show();
+			$('#row_basic_header').hide();
+			$('#row_basic_auth_fail_message').hide();
+			$('#row_path_basic_mapfile').hide();
+			$('#row_ldap_network_timeout').hide();
+			$('#row_ldap_bind_timeout').hide();
+			$('#row_ldap_tls_certificate').hide();
+			$('#row_ldap_debug').hide();
 			break;
 		}
 	}
@@ -1241,13 +1304,3 @@ default:
 	break;
 }
 
-function is_remote_path_setting($field_name) {
-	global $config;
-
-	if ($config['poller_id'] > 1 && (strpos($field_name, 'path_') !== false || strpos($field_name, '_path') !== false)) {
-		return true;
-	} else {
-		return false;
-	}
-}
-
