@@ -1,8 +1,8 @@
-#!/usr/bin/php -q
+#!/usr/bin/env php
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2019 The Cacti Group                                 |
+ | Copyright (C) 2004-2022 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -33,7 +33,7 @@ if (!isset($called_by_script_server)) {
 	print call_user_func_array('ss_cpoller', $_SERVER['argv']);
 }
 
-function ss_cpoller($cmd, $arg1 = '', $arg2 = '') {
+function ss_cpoller($cmd = 'index', $arg1 = '', $arg2 = '') {
 	if ($cmd == 'index') {
 		$collectors = db_fetch_assoc('SELECT id FROM poller ORDER BY id');
 
@@ -160,4 +160,3 @@ function ss_cpoller($cmd, $arg1 = '', $arg2 = '') {
 		return ($value == '' ? '0' : $value);
 	}
 }
-
